@@ -1,4 +1,5 @@
 #!/bin/sh
+set -e
 
 # This is a script to be run as part of make.sh. The only time you'd
 # probably want to run it by itself is if you're cross-compiling the
@@ -14,19 +15,19 @@
 # files for more information.
 
 LANG=C
-export LANG
+LC_ALL=C
+export LANG LC_ALL
 
 . ./find-gnumake.sh
 find_gnumake
 
-# usually SBCL_HOME refers to the installed root of SBCL, not the
-# build directory.  Right now, however, where there are dependencies
-# between contrib packages, we want the _uninstalled_ versions to be
-# found
-SBCL_HOME=`pwd`/contrib
+. ./sbcl-pwd.sh
+sbcl_pwd
+
+SBCL_HOME=$SBCL_PWD/contrib
 export SBCL_HOME
 
-SBCL="`pwd`/src/runtime/sbcl --noinform --core `pwd`/output/sbcl.core --userinit /dev/null --sysinit /dev/null --disable-debugger"
+SBCL="$SBCL_PWD/src/runtime/sbcl --noinform --core $SBCL_PWD/output/sbcl.core --disable-debugger --no-sysinit --no-userinit"
 SBCL_BUILDING_CONTRIB=1
 export SBCL SBCL_BUILDING_CONTRIB
 
@@ -36,19 +37,37 @@ export SBCL SBCL_BUILDING_CONTRIB
 # as SB-RT and SB-GROVEL, but FIXME: there's probably a better
 # solution.  -- CSR, 2003-05-30
 
-find contrib/ \( -name '*.fasl' -o -name 'foo.c' -o -name 'a.out' -o -name 'alien.so' -o -name '*.o' \) \
+find contrib/ \( -name '*.fasl' -o \
+                 -name '*.FASL' -o \
+                 -name 'foo.c' -o \
+                 -name 'FOO.C' -o \
+                 -name 'a.out' -o \
+                 -name 'A.OUT' -o \
+                 -name 'alien.so' -o \
+                 -name 'ALIEN.SO' -o \
+                 -name '*.o' -o \
+                 -name '*.O' \) \
   -print | xargs rm -f
 
-mkdir -p contrib/systems
-rm -f contrib/systems/*
-
-for i in contrib/*/*.asd; do
-    ln -sf ../../$i contrib/systems/
-done
-
 for i in contrib/*; do
     test -d $i && test -f $i/Makefile || continue;
     # export INSTALL_DIR=$SBCL_HOME/`basename $i `
-    test -f $i/test-passed && rm $i/test-passed 
+    test -f $i/test-passed && rm $i/test-passed
     $GNUMAKE -C $i test && touch $i/test-passed
 done
+
+# Sometimes people used to see the "No tests failed." output from the last
+# DEFTEST in contrib self-tests and think that's all that is. So...
+HEADER_HAS_BEEN_PRINTED=false
+for dir in contrib/*
+do
+  if [ -d "$dir" -a -f "$dir/Makefile" -a ! -f "$dir/test-passed" ]; then
+      if $HEADER_HAS_BEEN_PRINTED; then
+          echo > /dev/null
+      else
+          echo "Failed contribs:"
+          HEADER_HAS_BEEN_PRINTED=true
+      fi
+      echo "  `basename $dir`"
+  fi
+done
