@@ -35,38 +35,38 @@
 
 (macrolet ((define-fasl-format-features ()
              (let (;; master value for *F-P-A-F-F*
-		   (fpaff '(:sb-thread :sb-package-locks :sb-unicode)))
-	       `(progn
-		  ;; a list of *(SHEBANG-)FEATURES* flags which affect
-		  ;; binary compatibility, i.e. which must be the same
-		  ;; between the SBCL which compiles the code and the
-		  ;; SBCL which executes the code
-		  ;;
-		  ;; This is a property of SBCL executables in the
-		  ;; abstract, not of this particular SBCL executable, 
-		  ;; so any flag in this list may or may not be present
-		  ;; in the *FEATURES* list of this particular build.
-		  (defparameter *features-potentially-affecting-fasl-format*
-		    ',fpaff)
-		  ;; a string representing flags of *F-P-A-F-F* which
-		  ;; are in this particular build
-		  ;;
-		  ;; (A list is the natural logical representation for
-		  ;; this, but we represent it as a string because
-		  ;; that's physically convenient for writing to and
-		  ;; reading from fasl files, and because we don't
-		  ;; need to do anything sophisticated with its
-		  ;; logical structure, just test it for equality.)
-		  (defparameter *features-affecting-fasl-format*
-		    ,(let ((*print-pretty* nil))
-		       (prin1-to-string
-			(sort
-			 (copy-seq
-			  (intersection sb-cold:*shebang-features* fpaff))
-			 #'string<
-			 :key #'symbol-name))))))))
+                   (fpaff '(:sb-thread :sb-package-locks :sb-unicode :gencgc)))
+               `(progn
+                  ;; a list of *(SHEBANG-)FEATURES* flags which affect
+                  ;; binary compatibility, i.e. which must be the same
+                  ;; between the SBCL which compiles the code and the
+                  ;; SBCL which executes the code
+                  ;;
+                  ;; This is a property of SBCL executables in the
+                  ;; abstract, not of this particular SBCL executable,
+                  ;; so any flag in this list may or may not be present
+                  ;; in the *FEATURES* list of this particular build.
+                  (defparameter *features-potentially-affecting-fasl-format*
+                    ',fpaff)
+                  ;; a string representing flags of *F-P-A-F-F* which
+                  ;; are in this particular build
+                  ;;
+                  ;; (A list is the natural logical representation for
+                  ;; this, but we represent it as a string because
+                  ;; that's physically convenient for writing to and
+                  ;; reading from fasl files, and because we don't
+                  ;; need to do anything sophisticated with its
+                  ;; logical structure, just test it for equality.)
+                  (defparameter *features-affecting-fasl-format*
+                    ,(let ((*print-pretty* nil))
+                       (prin1-to-string
+                        (sort
+                         (copy-seq
+                          (intersection sb-cold:*shebang-features* fpaff))
+                         #'string<
+                         :key #'symbol-name))))))))
   (define-fasl-format-features))
-	   
+
 ;;; the code for a character which terminates a fasl file header
 (def!constant +fasl-header-string-stop-char-code+ 255)
 
@@ -76,7 +76,7 @@
 ;;; versions which break binary compatibility. But it certainly should
 ;;; be incremented for release versions which break binary
 ;;; compatibility.
-(def!constant +fasl-file-version+ 57)
+(def!constant +fasl-file-version+ 66)
 ;;; (record of versions before 2003 deleted in 2003-04-26/0.pre8.107 or so)
 ;;; 38: (2003-01-05) changed names of internal SORT machinery
 ;;; 39: (2003-02-20) in 0.7.12.1 a slot was added to
@@ -95,10 +95,10 @@
 ;;;     <Krystof_> I think I renumbered everything again
 ;;;     <Krystof_> simple-array-unsigned-byte-7, probably
 ;;;     <Krystof_> (thanks to pfdietz)
-;;; 45: (2003-10-02) I (WHN) incremented the version for the 0.8.4 
+;;; 45: (2003-10-02) I (WHN) incremented the version for the 0.8.4
 ;;;     release because I couldn't immediately convince myself that
 ;;;     .fasl files could never possibly ever refer to the SB-C
-;;;     CONTINUATION-related data types which were changed 
+;;;     CONTINUATION-related data types which were changed
 ;;;     incompatibly in 0.8.3.62.
 ;;; 46: (2003-11-11) Tim Daly, Jr. (and Christophe Rhodes) reported
 ;;;     .fasl incompatibility on sbcl-devel 2003-11-09.
@@ -122,6 +122,20 @@
 ;;; 56: (2005-05-22) Something between 0.9.0.1 and 0.9.0.14. My money is
 ;;;     on 0.9.0.6 (MORE CASE CONSISTENCY).
 ;;; 57: (2005-06-12) Raw slot rearrangement in 0.9.1.38
+;;; 58: (2005-08-16) Multiple incompatible changes between 0.9.3 and 0.9.3.60
+;;; 59: (2005-09-18) METAOBJECT implementation, removal of INSTANCE and
+;;;     FUNCALLABLE-INSTANCE classes.
+;;; 60: (2005-10-24) Bumped for 0.9.6
+;;; 61: (2005-11-06) Improved source location recording added extra parameters
+;;;     to multiple %DEFMUMBLE functions.
+;;; 62: (2005-12-30) Make the count of FASL header counted strings
+;;;     a 32-bit value also on 64-bit platforms.
+;;; 63: (2006-01-27) Shuffle storage classes around to reduce the error
+;;;     trap information size on RISCy platforms.
+;;; 64: (2006-03-24) New calling convention for unknown-values on x86 and
+;;;     x86-64.  Also (belatedly) PPC/gencgc, including :gencgc on FPAFF.
+;;; 65: (2006-04-11) Package locking interface changed.
+;;; 66: (2006-05-13) Fopcompiler
 
 ;;; the conventional file extension for our fasl files
 (declaim (type simple-string *fasl-file-type*))
@@ -148,8 +162,8 @@
 ;;; which will perform the operation
 (defvar *fop-funs*
   (make-array 256
-	      :initial-element (lambda ()
-				 (error "corrupt fasl file: losing FOP"))))
+              :initial-element (lambda ()
+                                 (error "corrupt fasl file: losing FOP"))))
 
 ;;;; variables
 
