@@ -12,13 +12,8 @@
 
 (in-package "SB!KERNEL")
 (define-alien-routine ("protect_control_stack_guard_page"
-		       %protect-control-stack-guard-page)
+                       %protect-control-stack-guard-page)
     sb!alien:void
-  (thread-id #!+sb-thread sb!alien:unsigned-long
-             #!-sb-thread sb!alien:int)
   (protect-p sb!alien:int))
 (defun protect-control-stack-guard-page (n)
-  (%protect-control-stack-guard-page 
-   (sb!thread:current-thread-id) (if n 1 0)))
-
-
+  (%protect-control-stack-guard-page (if n 1 0)))
