@@ -18,28 +18,40 @@
 ;;; translation the inverse (unparse) functions.
 (def!struct (host (:constructor nil))
   (parse (missing-arg) :type function)
+  (parse-native (missing-arg) :type function)
   (unparse (missing-arg) :type function)
+  (unparse-native (missing-arg) :type function)
   (unparse-host (missing-arg) :type function)
   (unparse-directory (missing-arg) :type function)
   (unparse-file (missing-arg) :type function)
   (unparse-enough (missing-arg) :type function)
+  (unparse-directory-separator (missing-arg) :type simple-string)
   (customary-case (missing-arg) :type (member :upper :lower)))
 
 (def!method print-object ((host host) stream)
   (print-unreadable-object (host stream :type t :identity t)))
 
 (def!struct (logical-host
-	     (:make-load-form-fun make-logical-host-load-form-fun)
-	     (:include host
-		       (parse #'parse-logical-namestring)
-		       (unparse #'unparse-logical-namestring)
-		       (unparse-host
-			(lambda (x)
-			  (logical-host-name (%pathname-host x))))
-		       (unparse-directory #'unparse-logical-directory)
-		       (unparse-file #'unparse-logical-file)
-		       (unparse-enough #'unparse-enough-namestring)
-		       (customary-case :upper)))
+             (:make-load-form-fun make-logical-host-load-form-fun)
+             (:include host
+                       (parse #'parse-logical-namestring)
+                       (parse-native
+                        (lambda (x)
+                          (error "called PARSE-NATIVE-NAMESTRING using a ~
+                                  logical host: ~S" x)))
+                       (unparse #'unparse-logical-namestring)
+                       (unparse-native
+                        (lambda (x)
+                          (error "called NATIVE-NAMESTRING using a ~
+                                  logical host: ~S" x)))
+                       (unparse-host
+                        (lambda (x)
+                          (logical-host-name (%pathname-host x))))
+                       (unparse-directory #'unparse-logical-directory)
+                       (unparse-file #'unparse-logical-file)
+                       (unparse-enough #'unparse-enough-namestring)
+                       (unparse-directory-separator ";")
+                       (customary-case :upper)))
   (name "" :type simple-base-string)
   (translations nil :type list)
   (canon-transls nil :type list))
@@ -50,7 +62,7 @@
 
 (defun make-logical-host-load-form-fun (logical-host)
   (values `(find-logical-host ',(logical-host-name logical-host))
-	  nil))
+          nil))
 
 ;;; A PATTERN is a list of entries and wildcards used for pattern
 ;;; matches of translations.
@@ -66,13 +78,13 @@
     '(member nil :unspecific :wild)))
 
 (sb!xc:defstruct (pathname (:conc-name %pathname-)
-			   (:constructor %make-pathname (host
-							 device
-							 directory
-							 name
-							 type
-							 version))
-			   (:predicate pathnamep))
+                           (:constructor %make-pathname (host
+                                                         device
+                                                         directory
+                                                         name
+                                                         type
+                                                         version))
+                           (:predicate pathnamep))
   ;; the host (at present either a UNIX or logical host)
   (host nil :type (or host null))
   ;; the name of a logical or physical device holding files
@@ -90,7 +102,7 @@
 ;;; Logical pathnames have the following format:
 ;;;
 ;;; logical-namestring ::=
-;;;	 [host ":"] [";"] {directory ";"}* [name] ["." type ["." version]]
+;;;      [host ":"] [";"] {directory ";"}* [name] ["." type ["." version]]
 ;;;
 ;;; host ::= word
 ;;; directory ::= word | wildcard-word | **
@@ -106,11 +118,11 @@
 ;;; Logical pathnames are a subclass of PATHNAME. Their class
 ;;; relations are mimicked using structures for efficiency.
 (sb!xc:defstruct (logical-pathname (:conc-name %logical-pathname-)
-				   (:include pathname)
-				   (:constructor %make-logical-pathname
-						 (host
-						  device
-						  directory
-						  name
-						  type
-						  version))))
+                                   (:include pathname)
+                                   (:constructor %make-logical-pathname
+                                                 (host
+                                                  device
+                                                  directory
+                                                  name
+                                                  type
+                                                  version))))
