@@ -19,20 +19,29 @@
     (declare (optimize (safety 0) (speed 3)))
     (sb!vm::locked-symbol-global-value-add ',symbol-name ,delta)))
 
-;;; When >0, inhibits garbage collection.
-(declaim (type index *gc-inhibit*))
 (defvar *gc-inhibit*) ; initialized in cold init
 
+;;; When the dynamic usage increases beyond this amount, the system
+;;; notes that a garbage collection needs to occur by setting
+;;; *GC-PENDING* to T. It starts out as NIL meaning nobody has figured
+;;; out what it should be yet.
+(defvar *gc-pending* nil)
+
+#!+sb-thread
+(defvar *stop-for-gc-pending* nil)
+
 (defmacro without-gcing (&body body)
   #!+sb-doc
-  "Executes the forms in the body without doing a garbage collection."
+  "Executes the forms in the body without doing a garbage
+collection. It inhibits both automatically and explicitly triggered
+gcs. Finally, upon leaving the BODY if gc is not inhibited it runs the
+pending gc. Similarly, if gc is triggered in another thread then it
+waits until gc is enabled in this thread."
   `(unwind-protect
-    (progn
-      (atomic-incf/symbol *gc-inhibit*)
-      ,@body)
-    (atomic-incf/symbol *gc-inhibit* -1)
-    (when (and *need-to-collect-garbage* (zerop *gc-inhibit*))
-      (sub-gc))))
+        (let ((*gc-inhibit* t))
+          ,@body)
+     ;; the test is racy, but it can err only on the overeager side
+     (sb!kernel::maybe-handle-pending-gc)))
 
 
 ;;; EOF-OR-LOSE is a useful macro that handles EOF.
@@ -49,30 +58,30 @@
   (let ((svar (gensym)))
     `(let ((,svar ,stream))
        (cond ((null ,svar) *standard-input*)
-	     ((eq ,svar t) *terminal-io*)
-	     (t ,@(when check-type `((enforce-type ,svar ,check-type)))	;
-		#!+high-security
-		(unless (input-stream-p ,svar)
-		  (error 'simple-type-error
-			 :datum ,svar
-			 :expected-type '(satisfies input-stream-p)
-			 :format-control "~S isn't an input stream"
-			 :format-arguments (list ,svar)))
-		,svar)))))
+             ((eq ,svar t) *terminal-io*)
+             (t ,@(when check-type `((enforce-type ,svar ,check-type))) ;
+                #!+high-security
+                (unless (input-stream-p ,svar)
+                  (error 'simple-type-error
+                         :datum ,svar
+                         :expected-type '(satisfies input-stream-p)
+                         :format-control "~S isn't an input stream"
+                         :format-arguments (list ,svar)))
+                ,svar)))))
 (defmacro out-synonym-of (stream &optional check-type)
   (let ((svar (gensym)))
     `(let ((,svar ,stream))
        (cond ((null ,svar) *standard-output*)
-	     ((eq ,svar t) *terminal-io*)
-	     (t ,@(when check-type `((check-type ,svar ,check-type)))
-		#!+high-security
-		(unless (output-stream-p ,svar)
-		  (error 'simple-type-error
-			 :datum ,svar
-			 :expected-type '(satisfies output-stream-p)
-			 :format-control "~S isn't an output stream."
-			 :format-arguments (list ,svar)))
-		,svar)))))
+             ((eq ,svar t) *terminal-io*)
+             (t ,@(when check-type `((check-type ,svar ,check-type)))
+                #!+high-security
+                (unless (output-stream-p ,svar)
+                  (error 'simple-type-error
+                         :datum ,svar
+                         :expected-type '(satisfies output-stream-p)
+                         :format-control "~S isn't an output stream."
+                         :format-arguments (list ,svar)))
+                ,svar)))))
 
 ;;; WITH-mumble-STREAM calls the function in the given SLOT of the
 ;;; STREAM with the ARGS for ANSI-STREAMs, or the FUNCTION with the
@@ -80,35 +89,43 @@
 (defmacro with-in-stream (stream (slot &rest args) &optional stream-dispatch)
   `(let ((stream (in-synonym-of ,stream)))
     ,(if stream-dispatch
-	 `(if (ansi-stream-p stream)
-	      (funcall (,slot stream) stream ,@args)
-	      ,@(when stream-dispatch
-		  `(,(destructuring-bind (function &rest args) stream-dispatch
-		       `(,function stream ,@args)))))
-	 `(funcall (,slot stream) stream ,@args))))
+         `(if (ansi-stream-p stream)
+              (funcall (,slot stream) stream ,@args)
+              ,@(when stream-dispatch
+                  `(,(destructuring-bind (function &rest args) stream-dispatch
+                       `(,function stream ,@args)))))
+         `(funcall (,slot stream) stream ,@args))))
+
+(defmacro with-out-stream/no-synonym (stream (slot &rest args) &optional stream-dispatch)
+  `(let ((stream ,stream))
+    ,(if stream-dispatch
+         `(if (ansi-stream-p stream)
+              (funcall (,slot stream) stream ,@args)
+              ,@(when stream-dispatch
+                  `(,(destructuring-bind (function &rest args) stream-dispatch
+                                         `(,function stream ,@args)))))
+         `(funcall (,slot stream) stream ,@args))))
 
 (defmacro with-out-stream (stream (slot &rest args) &optional stream-dispatch)
-  `(let ((stream (out-synonym-of ,stream)))
-    ,(if stream-dispatch
-	 `(if (ansi-stream-p stream)
-	      (funcall (,slot stream) stream ,@args)
-	      ,@(when stream-dispatch
-		  `(,(destructuring-bind (function &rest args) stream-dispatch
-					 `(,function stream ,@args)))))
-	 `(funcall (,slot stream) stream ,@args))))
+  `(with-out-stream/no-synonym (out-synonym-of ,stream)
+    (,slot ,@args) ,stream-dispatch))
+
 
 ;;;; These are hacks to make the reader win.
 
 ;;; This macro sets up some local vars for use by the
 ;;; FAST-READ-CHAR macro within the enclosed lexical scope. The stream
 ;;; is assumed to be a ANSI-STREAM.
+;;;
+;;; KLUDGE: Some functions (e.g. ANSI-STREAM-READ-LINE) use these variables
+;;; directly, instead of indirecting through FAST-READ-CHAR.
 (defmacro prepare-for-fast-read-char (stream &body forms)
   `(let* ((%frc-stream% ,stream)
-	  (%frc-method% (ansi-stream-in %frc-stream%))
-	  (%frc-buffer% (ansi-stream-cin-buffer %frc-stream%))
-	  (%frc-index% (ansi-stream-in-index %frc-stream%)))
+          (%frc-method% (ansi-stream-in %frc-stream%))
+          (%frc-buffer% (ansi-stream-cin-buffer %frc-stream%))
+          (%frc-index% (ansi-stream-in-index %frc-stream%)))
      (declare (type index %frc-index%)
-	      (type ansi-stream %frc-stream%))
+              (type ansi-stream %frc-stream%))
      ,@forms))
 
 ;;; This macro must be called after one is done with FAST-READ-CHAR
@@ -117,17 +134,17 @@
   `(setf (ansi-stream-in-index %frc-stream%) %frc-index%))
 
 ;;; a macro with the same calling convention as READ-CHAR, to be used
-;;; within the scope of a PREPARE-FOR-FAST-READ-CHAR
+;;; within the scope of a PREPARE-FOR-FAST-READ-CHAR.
 (defmacro fast-read-char (&optional (eof-error-p t) (eof-value ()))
   `(cond
     ((not %frc-buffer%)
      (funcall %frc-method% %frc-stream% ,eof-error-p ,eof-value))
     ((= %frc-index% +ansi-stream-in-buffer-length+)
      (prog1 (fast-read-char-refill %frc-stream% ,eof-error-p ,eof-value)
-	    (setq %frc-index% (ansi-stream-in-index %frc-stream%))))
+            (setq %frc-index% (ansi-stream-in-index %frc-stream%))))
     (t
      (prog1 (aref %frc-buffer% %frc-index%)
-	    (incf %frc-index%)))))
+            (incf %frc-index%)))))
 
 ;;;; And these for the fasloader...
 
@@ -143,11 +160,11 @@
 ;;; for the FAST-READ-CHAR stuff) -- WHN 19990825
 (defmacro prepare-for-fast-read-byte (stream &body forms)
   `(let* ((%frc-stream% ,stream)
-	  (%frc-method% (ansi-stream-bin %frc-stream%))
-	  (%frc-buffer% (ansi-stream-in-buffer %frc-stream%))
-	  (%frc-index% (ansi-stream-in-index %frc-stream%)))
+          (%frc-method% (ansi-stream-bin %frc-stream%))
+          (%frc-buffer% (ansi-stream-in-buffer %frc-stream%))
+          (%frc-index% (ansi-stream-in-index %frc-stream%)))
      (declare (type index %frc-index%)
-	      (type ansi-stream %frc-stream%))
+              (type ansi-stream %frc-stream%))
      ,@forms))
 
 ;;; Similar to fast-read-char, but we use a different refill routine & don't
@@ -162,9 +179,9 @@
       (funcall %frc-method% %frc-stream% ,eof-error-p ,eof-value))
      ((= %frc-index% +ansi-stream-in-buffer-length+)
       (prog1 (fast-read-byte-refill %frc-stream% ,eof-error-p ,eof-value)
-	(setq %frc-index% (ansi-stream-in-index %frc-stream%))))
+        (setq %frc-index% (ansi-stream-in-index %frc-stream%))))
      (t
       (prog1 (aref %frc-buffer% %frc-index%)
-	(incf %frc-index%))))))
+        (incf %frc-index%))))))
 (defmacro done-with-fast-read-byte ()
   `(done-with-fast-read-char))
