@@ -16,44 +16,44 @@
 ;;; Simple TYPEP uses that don't have any standard predicate are
 ;;; translated into non-standard unary predicates.
 (defknown (fixnump bignump ratiop
-	   short-float-p single-float-p double-float-p long-float-p
-	   complex-rational-p complex-float-p complex-single-float-p
-	   complex-double-float-p #!+long-float complex-long-float-p
-	   complex-vector-p
-	   base-char-p %standard-char-p %instancep
-	   base-string-p simple-base-string-p
+           short-float-p single-float-p double-float-p long-float-p
+           complex-rational-p complex-float-p complex-single-float-p
+           complex-double-float-p #!+long-float complex-long-float-p
+           complex-vector-p
+           base-char-p %standard-char-p %instancep
+           base-string-p simple-base-string-p
            #!+sb-unicode character-string-p
            #!+sb-unicode simple-character-string-p
-	   array-header-p
-	   simple-array-p simple-array-nil-p vector-nil-p
-	   simple-array-unsigned-byte-2-p
-	   simple-array-unsigned-byte-4-p simple-array-unsigned-byte-7-p
-	   simple-array-unsigned-byte-8-p simple-array-unsigned-byte-15-p
-	   simple-array-unsigned-byte-16-p
+           array-header-p
+           simple-array-p simple-array-nil-p vector-nil-p
+           simple-array-unsigned-byte-2-p
+           simple-array-unsigned-byte-4-p simple-array-unsigned-byte-7-p
+           simple-array-unsigned-byte-8-p simple-array-unsigned-byte-15-p
+           simple-array-unsigned-byte-16-p
            #!+#.(cl:if (cl:= 32 sb!vm:n-word-bits) '(and) '(or))
            simple-array-unsigned-byte-29-p
-	   simple-array-unsigned-byte-31-p
-	   simple-array-unsigned-byte-32-p
+           simple-array-unsigned-byte-31-p
+           simple-array-unsigned-byte-32-p
            #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
            simple-array-unsigned-byte-60-p
            #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
            simple-array-unsigned-byte-63-p
            #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
            simple-array-unsigned-byte-64-p
-	   simple-array-signed-byte-8-p simple-array-signed-byte-16-p
+           simple-array-signed-byte-8-p simple-array-signed-byte-16-p
            #!+#.(cl:if (cl:= 32 sb!vm:n-word-bits) '(and) '(or))
-	   simple-array-signed-byte-30-p
+           simple-array-signed-byte-30-p
            simple-array-signed-byte-32-p
            #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
            simple-array-signed-byte-61-p
            #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
            simple-array-signed-byte-64-p
-	   simple-array-single-float-p simple-array-double-float-p
-	   #!+long-float simple-array-long-float-p
-	   simple-array-complex-single-float-p
-	   simple-array-complex-double-float-p
-	   #!+long-float simple-array-complex-long-float-p
-	   system-area-pointer-p realp
+           simple-array-single-float-p simple-array-double-float-p
+           #!+long-float simple-array-long-float-p
+           simple-array-complex-single-float-p
+           simple-array-complex-double-float-p
+           #!+long-float simple-array-complex-long-float-p
+           system-area-pointer-p realp
            ;; #!+#.(cl:if (cl:= 32 sb!vm:n-word-bits) '(and) '(or))
            unsigned-byte-32-p
            ;; #!+#.(cl:if (cl:= 32 sb!vm:n-word-bits) '(and) '(or))
@@ -62,8 +62,8 @@
            unsigned-byte-64-p
            #!+#.(cl:if (cl:= 64 sb!vm:n-word-bits) '(and) '(or))
            signed-byte-64-p
-	   vector-t-p weak-pointer-p code-component-p lra-p
-	   funcallable-instance-p)
+           vector-t-p weak-pointer-p code-component-p lra-p
+           funcallable-instance-p)
   (t) boolean (movable foldable flushable))
 
 ;;;; miscellaneous "sub-primitives"
@@ -116,44 +116,53 @@
 (defknown %instance-length (instance) index
   (foldable flushable))
 (defknown %instance-ref (instance index) t
-  (flushable))
+  (flushable always-translatable))
 (defknown %instance-set (instance index t) t
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %layout-invalid-error (t layout) nil)
 
+#!-hppa
+(progn
 (defknown %raw-instance-ref/word (instance index) sb!vm:word
-  (flushable))
+  (flushable always-translatable))
 (defknown %raw-instance-set/word (instance index sb!vm:word) sb!vm:word
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %raw-instance-ref/single (instance index) single-float
-  (flushable))
+  (flushable always-translatable))
 (defknown %raw-instance-set/single (instance index single-float) single-float
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %raw-instance-ref/double (instance index) double-float
-  (flushable))
+  (flushable always-translatable))
 (defknown %raw-instance-set/double (instance index double-float) double-float
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %raw-instance-ref/complex-single (instance index)
   (complex single-float)
-  (flushable))
+  (flushable always-translatable))
 (defknown %raw-instance-set/complex-single
     (instance index (complex single-float))
   (complex single-float)
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %raw-instance-ref/complex-double (instance index)
   (complex double-float)
-  (flushable))
+  (flushable always-translatable))
 (defknown %raw-instance-set/complex-double
     (instance index (complex double-float))
   (complex double-float)
-  (unsafe))
-
-(sb!xc:deftype raw-vector () '(simple-array sb!vm:word (*)))
+  (unsafe always-translatable))
+)
 
 ;;; %RAW-{REF,SET}-FOO VOPs should be declared as taking a RAW-VECTOR
 ;;; as their first argument (clarity and to match these DEFKNOWNs).
 ;;; We declare RAW-VECTOR as a primitive type so the VOP machinery
 ;;; will accept our VOPs as legitimate.  --njf, 2004-08-10
+;;;
+;;; These are only used on HPPA, since on HPPA implements raw slots in
+;;; structures with an indirection vector; all other ports implement
+;;; raw slots directly in the structure.  --njf, 2006-06-02
+#!+hppa
+(progn
+(sb!xc:deftype raw-vector () '(simple-array sb!vm:word (*)))
+
 (sb!vm::!def-primitive-type-alias raw-vector
                                   #!+#.(cl:if (cl:= 32 sb!vm:n-word-bits) '(and) '(or))
                                   sb!vm::simple-array-unsigned-byte-32
@@ -161,32 +170,32 @@
                                   sb!vm::simple-array-unsigned-byte-64)
 
 (defknown %raw-ref-single (raw-vector index) single-float
-  (foldable flushable))
+  (foldable flushable always-translatable))
 (defknown %raw-ref-double (raw-vector index) double-float
-  (foldable flushable))
+  (foldable flushable always-translatable))
 #!+long-float
 (defknown %raw-ref-long (raw-vector index) long-float
-  (foldable flushable))
+  (foldable flushable always-translatable))
 (defknown %raw-set-single (raw-vector index single-float) single-float
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %raw-set-double (raw-vector index double-float) double-float
-  (unsafe))
+  (unsafe always-translatable))
 #!+long-float
 (defknown %raw-set-long (raw-vector index long-float) long-float
-  (unsafe))
+  (unsafe always-translatable))
 
 (defknown %raw-ref-complex-single (raw-vector index) (complex single-float)
-  (foldable flushable))
+  (foldable flushable always-translatable))
 (defknown %raw-ref-complex-double (raw-vector index) (complex double-float)
-  (foldable flushable))
+  (foldable flushable always-translatable))
 
 (defknown %raw-set-complex-single (raw-vector index (complex single-float))
   (complex single-float)
-  (unsafe))
+  (unsafe always-translatable))
 (defknown %raw-set-complex-double (raw-vector index (complex double-float))
   (complex double-float)
-  (unsafe))
-
+  (unsafe always-translatable))
+)
 
 (defknown %raw-bits (t fixnum) sb!vm:word
   (foldable flushable))
@@ -216,8 +225,15 @@
 (defknown make-value-cell (t) t
   (flushable movable))
 
+;;;; threading
+
+#!+(and sb-lutex sb-thread)
+(progn
+  (defknown sb!vm::%make-lutex () sb!vm::lutex ())
+  (defknown sb!vm::lutexp (t) boolean (foldable flushable)))
+
 (defknown (dynamic-space-free-pointer binding-stack-pointer-sap
-				      control-stack-pointer-sap)  ()
+                                      control-stack-pointer-sap)  ()
   system-area-pointer
   (flushable))
 
@@ -243,11 +259,11 @@
   (foldable flushable movable))
 
 (defknown (word-logical-and word-logical-nand
-	   word-logical-or word-logical-nor
-	   word-logical-xor word-logical-eqv
-	   word-logical-andc1 word-logical-andc2
-	   word-logical-orc1 word-logical-orc2)
-	  (sb!vm:word sb!vm:word) sb!vm:word
+           word-logical-or word-logical-nor
+           word-logical-xor word-logical-eqv
+           word-logical-andc1 word-logical-andc2
+           word-logical-orc1 word-logical-orc2)
+          (sb!vm:word sb!vm:word) sb!vm:word
   (foldable flushable movable))
 
 (defknown (shift-towards-start shift-towards-end) (sb!vm:word fixnum)
@@ -276,13 +292,13 @@
   (foldable flushable movable))
 
 (defknown (%add-with-carry %subtract-with-borrow)
-	  (bignum-element-type bignum-element-type (mod 2))
+          (bignum-element-type bignum-element-type (mod 2))
   (values bignum-element-type (mod 2))
   (foldable flushable movable))
 
 (defknown %multiply-and-add
-	  (bignum-element-type bignum-element-type bignum-element-type
-			       &optional bignum-element-type)
+          (bignum-element-type bignum-element-type bignum-element-type
+                               &optional bignum-element-type)
   (values bignum-element-type bignum-element-type)
   (foldable flushable movable))
 
@@ -309,7 +325,7 @@
   (foldable flushable movable))
 
 (defknown (%ashl %ashr %digit-logical-shift-right)
-	  (bignum-element-type (mod #.sb!vm:n-word-bits)) bignum-element-type
+          (bignum-element-type (mod #.sb!vm:n-word-bits)) bignum-element-type
   (foldable flushable movable))
 
 ;;;; bit-bashing routines
@@ -360,7 +376,7 @@
 (defknown fun-subtype (function) (unsigned-byte #.sb!vm:n-widetag-bits)
   (flushable))
 (defknown ((setf fun-subtype))
-	  ((unsigned-byte #.sb!vm:n-widetag-bits) function)
+          ((unsigned-byte #.sb!vm:n-widetag-bits) function)
   (unsigned-byte #.sb!vm:n-widetag-bits)
   ())
 
@@ -382,7 +398,7 @@
 (defknown %closure-index-ref (function index) t
   (flushable))
 
-(defknown %make-funcallable-instance (index layout) function
+(defknown %make-funcallable-instance (index) function
   (unsafe))
 
 (defknown %funcallable-instance-info (function index) t (flushable))
@@ -394,4 +410,4 @@
 
 (defknown %data-vector-and-index (array index)
                                  (values (simple-array * (*)) index)
-				 (foldable flushable))
+                                 (foldable flushable))
