@@ -27,7 +27,7 @@
           (t 3))
   ("no" "maybe" "yes" "yes"))
 
-(define-optimization-quality let-convertion
+(define-optimization-quality let-conversion
     (if (<= debug speed) 3 0)
   ("off" "maybe" "on" "on"))
 
@@ -56,9 +56,9 @@
 
 (define-optimization-quality stack-allocate-dynamic-extent
     (if (and (> (max speed space) (max debug safety))
-	     (< safety 3))
-	3
-	0)
+             (< safety 3))
+        3
+        0)
   ("no" "maybe" "yes" "yes"))
 
 (define-optimization-quality stack-allocate-vector
@@ -76,3 +76,7 @@
         debug
         0)
   ("no" "no" "partial" "full"))
+
+(define-optimization-quality compute-debug-fun
+    debug
+  ("no" "minimal" "yes" "yes"))
