@@ -1,6 +1,6 @@
 ;;;; This file contains some parameterizations of various VM
-;;;; attributes for the PPC.  This file is separate from other stuff so 
-;;;; that it can be compiled and loaded earlier. 
+;;;; attributes for the PPC.  This file is separate from other stuff so
+;;;; that it can be compiled and loaded earlier.
 
 ;;;; This software is part of the SBCL system. See the README file for
 ;;;; more information.
@@ -25,6 +25,14 @@
 (def!constant n-byte-bits 8)
 
 
+;;; The size in bytes of the GENCGC pages. Should be a multiple of the
+;;; architecture code size.
+(def!constant gencgc-page-size 4096)
+
+;;; flags for the generational garbage collector
+(def!constant pseudo-atomic-interrupted-flag 1)
+(def!constant pseudo-atomic-flag 4)
+
 (def!constant float-sign-shift 31)
 
 (def!constant single-float-bias 126)
@@ -61,14 +69,14 @@
 (def!constant float-round-to-positive 2)
 (def!constant float-round-to-negative 3)
 
-(defconstant-eqx float-rounding-mode (byte 2 0) #'equalp)	  ; RD
+(defconstant-eqx float-rounding-mode (byte 2 0) #'equalp)         ; RD
 ;;; FIXME I: Beware, all ye who trespass here. Despite its name,
 ;;; FLOAT-STICKY-BITS is not the byte specifier for sticky bits in the
 ;;; floating point control word. It is more like "accrued exceptions"
 ;;; where FLOAT-EXCEPTIONS-BYTE is "current exceptions". Consequently,
 ;;; on architectures where there is no "current exceptions"
 ;;; FLOAT-EXCEPTIONS-BYTE and FLOAT-STICKY-BITS had better be the
-;;; same. 
+;;; same.
 ;;;
 ;;; FIXME II: So, I've now documented this in comments in the PowerPC
 ;;; tree. This may not make it easy to find for when new architectures
@@ -77,9 +85,9 @@
 ;;; CSR, 2002-06-11
 (defconstant-eqx float-sticky-bits (byte 5 25) #'equalp)
 (defconstant-eqx float-traps-byte (byte 5 3) #'equalp)
-(defconstant-eqx float-exceptions-byte (byte 5 25) #'equalp)	  ; cexc
+(defconstant-eqx float-exceptions-byte (byte 5 25) #'equalp)      ; cexc
 
-(def!constant float-fast-bit 2)		; Non-IEEE mode
+(def!constant float-fast-bit 2)         ; Non-IEEE mode
 
 
 ;;; Where to put the different spaces.
@@ -90,14 +98,38 @@
 (def!constant static-space-start    #x08000000)
 (def!constant static-space-end      #x097fff00)
 
-;;; nothing _seems_ to be using these addresses 
-(def!constant dynamic-0-space-start #x10000000)
-(def!constant dynamic-0-space-end   #x3ffff000)
-(def!constant dynamic-1-space-start #x40000000)
-(def!constant dynamic-1-space-end   #x6ffff000)
+#!+linux
+(progn
+  #!+gencgc
+  (progn
+    (def!constant dynamic-space-start #x4f000000)
+    (def!constant dynamic-space-end   #x7efff000))
+  #!-gencgc
+  (progn
+    (def!constant dynamic-0-space-start #x4f000000)
+    (def!constant dynamic-0-space-end   #x66fff000)
+    (def!constant dynamic-1-space-start #x67000000)
+    (def!constant dynamic-1-space-end   #x7efff000))
+
+  (def!constant linkage-table-space-start #x0a000000)
+  (def!constant linkage-table-space-end   #x0b000000)
+  (def!constant linkage-table-entry-size 16))
 
 #!+darwin
 (progn
+  #!+gencgc
+  (progn
+    (def!constant dynamic-space-start #x10000000)
+    (def!constant dynamic-space-end   #x6ffff000))
+  #!-gencgc
+  (progn
+    (def!constant dynamic-0-space-start #x10000000)
+    (def!constant dynamic-0-space-end   #x3ffff000)
+
+    (def!constant dynamic-1-space-start #x40000000)
+    (def!constant dynamic-1-space-end   #x6ffff000))
+
+
   (def!constant linkage-table-space-start #x0a000000)
   (def!constant linkage-table-space-end   #x0b000000)
   (def!constant linkage-table-entry-size 16))
@@ -137,40 +169,17 @@
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
 (defparameter *static-symbols*
-  '(t
+  (append
+   *common-static-symbols*
+   *c-callable-static-symbols*
+   '(
+     #!+gencgc *restart-lisp-function*
 
-    ;; The C startup code must fill these in.
-    *posix-argv*
-
-    ;; functions that the C code needs to call
-    sb!impl::sub-gc
-    sb!kernel::internal-error
-    sb!kernel::control-stack-exhausted-error
-    sb!kernel::undefined-alien-variable-error
-    sb!kernel::undefined-alien-function-error
-    sb!di::handle-breakpoint
-    sb!impl::fdefinition-object
-
-    ;; free pointers
-    *read-only-space-free-pointer*
-    *static-space-free-pointer*
-    *initial-dynamic-space-free-pointer*
-
-    ;; things needed for non-local exit
-    *current-catch-block*
-    *current-unwind-protect-block*
-
-    *binding-stack-start*
-    *control-stack-start*
-    *control-stack-end*
-
-    ;; interrupt handling
-    *free-interrupt-context-index*
-    sb!unix::*interrupts-enabled*
-    sb!unix::*interrupt-pending*
-
-
-    ))
+     ;; CLH: 20060210 Taken from x86-64/parms.lisp per JES' suggestion
+     ;; Needed for callbacks to work across saving cores. see
+     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
+     ;; details.
+     sb!alien::*enter-alien-callback*)))
 
 (defparameter *static-funs*
   '(length
@@ -182,7 +191,7 @@
     sb!kernel:two-arg->
     sb!kernel:two-arg-=
     sb!kernel:two-arg-<=
-    sb!kernel:two-arg->=   
+    sb!kernel:two-arg->=
     sb!kernel:two-arg-/=
     eql
     sb!kernel:%negate
