@@ -2,7 +2,7 @@
 ;;; Written by Rob MacLachlan
 ;;;
 ;;; Converted for SPARC by William Lott.
-;;; 
+;;;
 
 (in-package "SB!VM")
 
@@ -11,26 +11,6 @@
   (:generator 1
     (move csp-tn ptr)))
 
-(define-vop (%%pop-dx)
-  (:args (ptr :scs (any-reg)))
-  (:ignore ptr)
-  (:generator 1
-    (bug "VOP %%POP-DX is not implemented.")))
-
-(define-vop (%%nip-dx)
-  (:args (last-nipped-ptr :scs (any-reg) :target dest)
-	 (last-preserved-ptr :scs (any-reg) :target src)
-	 (moved-ptrs :scs (any-reg) :more t))
-  (:results (r-moved-ptrs :scs (any-reg) :more t))
-  (:temporary (:sc any-reg) src)
-  (:temporary (:sc any-reg) dest)
-  (:temporary (:sc non-descriptor-reg) temp)
-  (:ignore r-moved-ptrs
-           last-nipped-ptr last-preserved-ptr moved-ptrs
-           src dest temp)
-  (:generator 1
-    (bug "VOP %%NIP-DX is not implemented.")))
-
 ;;; sparc version translated to ppc by David Steuber with help from #lisp.
 (define-vop (%%nip-values)
   (:args (last-nipped-ptr :scs (any-reg) :target dest)
@@ -78,22 +58,22 @@
 (define-vop (push-values)
   (:args (vals :more t))
   (:results (start :scs (any-reg) :from :load)
-	    (count :scs (any-reg)))
+            (count :scs (any-reg)))
   (:info nvals)
   (:temporary (:scs (descriptor-reg)) temp)
   (:generator 20
     (inst mr start csp-tn)
     (inst addi csp-tn csp-tn (* nvals n-word-bytes))
     (do ((val vals (tn-ref-across val))
-	 (i 0 (1+ i)))
-	((null val))
+         (i 0 (1+ i)))
+        ((null val))
       (let ((tn (tn-ref-tn val)))
-	(sc-case tn
-	  (descriptor-reg
-	   (storew tn start i))
-	  (control-stack
-	   (load-stack-tn temp tn)
-	   (storew temp start i)))))
+        (sc-case tn
+          (descriptor-reg
+           (storew tn start i))
+          (control-stack
+           (load-stack-tn temp tn)
+           (storew temp start i)))))
     (inst lr count (fixnumize nvals))))
 
 ;;; Push a list of values on the stack, returning Start and Count as used in
@@ -104,7 +84,7 @@
   (:arg-types list)
   (:policy :fast-safe)
   (:results (start :scs (any-reg))
-	    (count :scs (any-reg)))
+            (count :scs (any-reg)))
   (:temporary (:scs (descriptor-reg) :type list :from (:argument 0)) list)
   (:temporary (:scs (descriptor-reg)) temp)
   (:temporary (:scs (non-descriptor-reg)) ndescr)
@@ -112,7 +92,7 @@
   (:save-p :compute-only)
   (:generator 0
     (let ((loop (gen-label))
-	  (done (gen-label)))
+          (done (gen-label)))
 
       (move list arg)
       (move start csp-tn)
@@ -136,15 +116,15 @@
 ;;;
 (define-vop (%more-arg-values)
   (:args (context :scs (descriptor-reg any-reg) :target src)
-	 (skip :scs (any-reg zero immediate))
-	 (num :scs (any-reg) :target count))
+         (skip :scs (any-reg zero immediate))
+         (num :scs (any-reg) :target count))
   (:arg-types * positive-fixnum positive-fixnum)
   (:temporary (:sc any-reg :from (:argument 0)) src)
   (:temporary (:sc any-reg :from (:argument 2)) dst)
   (:temporary (:sc descriptor-reg :from (:argument 1)) temp)
   (:temporary (:sc any-reg) i)
   (:results (start :scs (any-reg))
-	    (count :scs (any-reg)))
+            (count :scs (any-reg)))
   (:generator 20
     (sc-case skip
       (zero
