@@ -35,13 +35,17 @@
 ;;; addressable object
 (def!constant n-byte-bits 8)
 
+;;; The size in bytes of the GENCGC pages. Should be a multiple of the
+;;; architecture code size.
+(def!constant gencgc-page-size 4096)
+
 (def!constant float-sign-shift 31)
 
 ;;; comment from CMU CL:
 ;;;   These values were taken from the alpha code. The values for
 ;;;   bias and exponent min/max are not the same as shown in the 486 book.
 ;;;   They may be correct for how Python uses them.
-(def!constant single-float-bias 126)	; Intel says 127.
+(def!constant single-float-bias 126)    ; Intel says 127.
 (defconstant-eqx single-float-exponent-byte    (byte 8 23) #'equalp)
 (defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
 ;;; comment from CMU CL:
@@ -67,12 +71,12 @@
   (+ (byte-size double-float-significand-byte) 32 1))
 
 ;;; from AMD64 Architecture manual
-(def!constant float-invalid-trap-bit	   (ash 1 0))
+(def!constant float-invalid-trap-bit       (ash 1 0))
 (def!constant float-denormal-trap-bit       (ash 1 1))
 (def!constant float-divide-by-zero-trap-bit (ash 1 2))
 (def!constant float-overflow-trap-bit       (ash 1 3))
 (def!constant float-underflow-trap-bit      (ash 1 4))
-(def!constant float-inexact-trap-bit	   (ash 1 5))
+(def!constant float-inexact-trap-bit       (ash 1 5))
 
 (def!constant float-round-to-nearest  0)
 (def!constant float-round-to-negative 1)
@@ -89,6 +93,14 @@
 
 ;;; where to put the different spaces.
 
+;;; Currently the read-only and static spaces must be located in low
+;;; memory (certainly under the 4GB limit, very probably under 2GB
+;;; limit). This is due to the inability of using immediate values of
+;;; more than 32 bits (31 bits if you take sign extension into
+;;; account) in any other instructions except MOV. Removing this limit
+;;; would be possible, but probably not worth the time and code bloat
+;;; it would cause. -- JES, 2005-12-11
+
 (def!constant read-only-space-start     #x20000000)
 (def!constant read-only-space-end       #x27ff0000)
 
@@ -147,74 +159,47 @@
 (defvar *allocation-pointer*)
 (defvar *binding-stack-pointer*)
 
-;;; FIXME: !COLD-INIT probably doesn't need
-;;; to be in the static symbols table any more.
-;;;
-;;; FIXME: some of these symbols are shared by all backends,
-;;; and should be factored out into a common file.
 (defparameter *static-symbols*
-  '(t
+  (append
+   *common-static-symbols*
+   *c-callable-static-symbols*
+   '(*alien-stack*
 
-    ;; The C startup code must fill these in.
-    *posix-argv*
+     ;; interrupt handling
+     *pseudo-atomic-atomic*
+     *pseudo-atomic-interrupted*
 
-    ;; functions that the C code needs to call.  When adding to this list,
-    ;; also add a `frob' form in genesis.lisp finish-symbols.
-    sub-gc
-    sb!kernel::internal-error
-    sb!kernel::control-stack-exhausted-error
-    sb!kernel::undefined-alien-variable-error
-    sb!kernel::undefined-alien-function-error
-    sb!kernel::memory-fault-error
-    sb!di::handle-breakpoint
-    fdefinition-object
-    #!+sb-thread sb!thread::handle-thread-exit
+     #!+sb-thread *stop-for-gc-pending*
 
-    ;; free pointers
-    ;; 
-    ;; Note that these are FIXNUM word counts, not (as one might
-    ;; expect) byte counts or SAPs. The reason seems to be that by
-    ;; representing them this way, we can avoid consing bignums. 
-    ;; -- WHN 2000-10-02
-    *read-only-space-free-pointer*
-    *static-space-free-pointer*
-    *initial-dynamic-space-free-pointer*
+     #!+sb-thread *free-tls-index*
+     #!+sb-thread *tls-index-lock*
 
-    ;; things needed for non-local exit
-    *current-catch-block*
-    *current-unwind-protect-block*
-    *alien-stack*
+     *allocation-pointer*
+     *binding-stack-pointer*
 
-    ;; interrupt handling
-    *pseudo-atomic-atomic*
-    *pseudo-atomic-interrupted*
-    sb!unix::*interrupts-enabled*
-    sb!unix::*interrupt-pending*
-    *free-interrupt-context-index*
+     ;; the floating point constants
+     *fp-constant-0d0*
+     *fp-constant-1d0*
+     *fp-constant-0f0*
+     *fp-constant-1f0*
 
-    *free-tls-index*
-    
-    *allocation-pointer*
-    *binding-stack-pointer*
-    *binding-stack-start*
-    *control-stack-start*
-    *control-stack-end*
+     ;; For GC-AND-SAVE
+     *restart-lisp-function*
 
-    *need-to-collect-garbage*
+     ;; Needed for callbacks to work across saving cores. see
+     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
+     ;; details.
+     sb!alien::*enter-alien-callback*
 
-    ;; the floating point constants
-    *fp-constant-0d0*
-    *fp-constant-1d0*
-    *fp-constant-0f0*
-    *fp-constant-1f0*
-
-    ;; The ..SLOT-UNBOUND.. symbol is static in order to optimise the
-    ;; common slot unbound check.
-    ;;
-    ;; FIXME: In SBCL, the CLOS code has become sufficiently tightly
-    ;; integrated into the system that it'd probably make sense to use
-    ;; the ordinary unbound marker for this.
-    sb!pcl::..slot-unbound..))
+     ;; The ..SLOT-UNBOUND.. symbol is static in order to optimise the
+     ;; common slot unbound check.
+     ;;
+     ;; FIXME: In SBCL, the CLOS code has become sufficiently tightly
+     ;; integrated into the system that it'd probably make sense to
+     ;; use the ordinary unbound marker for this.
+     ;;
+     ;; FIXME II: if it doesn't make sense, why is this X86-ish only?
+     sb!pcl::..slot-unbound..)))
 
 (defparameter *static-funs*
   '(length
