@@ -35,13 +35,17 @@
 ;;; addressable object
 (def!constant n-byte-bits 8)
 
+;;; The size in bytes of the GENCGC pages. Should be a multiple of the
+;;; architecture code size.
+(def!constant gencgc-page-size 4096)
+
 (def!constant float-sign-shift 31)
 
 ;;; comment from CMU CL:
 ;;;   These values were taken from the alpha code. The values for
 ;;;   bias and exponent min/max are not the same as shown in the 486 book.
 ;;;   They may be correct for how Python uses them.
-(def!constant single-float-bias 126)	; Intel says 127.
+(def!constant single-float-bias 126)    ; Intel says 127.
 (defconstant-eqx single-float-exponent-byte    (byte 8 23) #'equalp)
 (defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
 ;;; comment from CMU CL:
@@ -65,7 +69,7 @@
 (defconstant-eqx long-float-significand-byte (byte 31 0) #'equalp)
 (def!constant long-float-normal-exponent-min 1)
 (def!constant long-float-normal-exponent-max #x7FFE)
-(def!constant long-float-hidden-bit (ash 1 31))		; actually not hidden
+(def!constant long-float-hidden-bit (ash 1 31))         ; actually not hidden
 (def!constant long-float-trapping-nan-bit (ash 1 30))
 
 (def!constant single-float-digits
@@ -78,12 +82,12 @@
   (+ (byte-size long-float-significand-byte) n-word-bits 1))
 
 ;;; pfw -- from i486 microprocessor programmer's reference manual
-(def!constant float-invalid-trap-bit	   (ash 1 0))
+(def!constant float-invalid-trap-bit       (ash 1 0))
 (def!constant float-denormal-trap-bit       (ash 1 1))
 (def!constant float-divide-by-zero-trap-bit (ash 1 2))
 (def!constant float-overflow-trap-bit       (ash 1 3))
 (def!constant float-underflow-trap-bit      (ash 1 4))
-(def!constant float-inexact-trap-bit	   (ash 1 5))
+(def!constant float-inexact-trap-bit       (ash 1 5))
 
 (def!constant float-round-to-nearest  0)
 (def!constant float-round-to-negative 1)
@@ -147,6 +151,21 @@
 ;;;     set the top-down mmap allocation option in the kernel (not yet
 ;;;     the default), all bets are totally off!
 
+#!+win32
+(progn
+
+  (def!constant read-only-space-start #x02000000)
+  (def!constant read-only-space-end   #x047ff000)
+
+  (def!constant static-space-start    #x05000000)
+  (def!constant static-space-end      #x07fff000)
+
+  (def!constant dynamic-space-start   #x09000000)
+  (def!constant dynamic-space-end     #x29000000)
+
+  (def!constant linkage-table-space-start #x30000000)
+  (def!constant linkage-table-space-end   #x40000000))
+
 #!+linux
 (progn
   (def!constant read-only-space-start     #x01000000)
@@ -161,6 +180,20 @@
   (def!constant linkage-table-space-start #x70000000)
   (def!constant linkage-table-space-end   #x7ffff000))
 
+#!+sunos
+(progn
+  (def!constant read-only-space-start     #x20000000)
+  (def!constant read-only-space-end       #x2ffff000)
+
+  (def!constant static-space-start        #x40000000)
+  (def!constant static-space-end          #x42fff000)
+
+  (def!constant dynamic-space-start       #x48000000)
+  (def!constant dynamic-space-end         #xA0000000)
+
+  (def!constant linkage-table-space-start #xA2000000)
+  (def!constant linkage-table-space-end   #xA3000000))
+
 #!+freebsd
 (progn
   (def!constant read-only-space-start     #x10000000)
@@ -206,6 +239,21 @@
   (def!constant linkage-table-space-start #xA0000000)
   (def!constant linkage-table-space-end   #xA1000000))
 
+
+#!+darwin
+(progn
+  (def!constant read-only-space-start #x01000000)
+  (def!constant read-only-space-end   #x04ff8000)
+
+  (def!constant static-space-start    #x08000000)
+  (def!constant static-space-end      #x097fff00)
+
+  (def!constant dynamic-space-start #x10000000)
+  (def!constant dynamic-space-end   #x6ffff000)
+
+  (def!constant linkage-table-space-start #x0a000000)
+  (def!constant linkage-table-space-end   #x0b000000))
+
 ;;; Size of one linkage-table entry in bytes.
 (def!constant linkage-table-entry-size 8)
 
@@ -222,7 +270,8 @@
   cerror
   breakpoint
   fun-end-breakpoint
-  single-step-breakpoint)
+  single-step-breakpoint
+  #!+win32 context-restore) ;; HACK: The Win32 exception handling system does wrong things with this.
 ;;; FIXME: It'd be nice to replace all the DEFENUMs with something like
 ;;;   (WITH-DEF-ENUM (:START 8)
 ;;;     (DEF-ENUM HALT-TRAP)
@@ -256,80 +305,47 @@
 (defvar *allocation-pointer*)
 (defvar *binding-stack-pointer*)
 
-;;; FIXME: !COLD-INIT probably doesn't need
-;;; to be in the static symbols table any more.
 (defparameter *static-symbols*
-  '(t
+  (append
+   *common-static-symbols*
+   *c-callable-static-symbols*
+   '(*alien-stack*
 
-    ;; The C startup code must fill these in.
-    *posix-argv*
+     ;; interrupt handling
+     *pseudo-atomic-atomic*
+     *pseudo-atomic-interrupted*
+     #!+sb-thread *stop-for-gc-pending*
+     #!+sb-thread *free-tls-index*
+     #!+sb-thread *tls-index-lock*
 
-    ;; functions that the C code needs to call.  When adding to this list,
-    ;; also add a `frob' form in genesis.lisp finish-symbols.
-    sub-gc
-    sb!kernel::internal-error
-    sb!kernel::control-stack-exhausted-error
-    sb!kernel::undefined-alien-variable-error
-    sb!kernel::undefined-alien-function-error
-    sb!kernel::memory-fault-error
-    sb!di::handle-breakpoint
-    fdefinition-object
-    #!+sb-thread sb!thread::handle-thread-exit
+     *allocation-pointer*
+     *binding-stack-pointer*
 
-    ;; free pointers
-    ;; 
-    ;; Note that these are FIXNUM word counts, not (as one might
-    ;; expect) byte counts or SAPs. The reason seems to be that by
-    ;; representing them this way, we can avoid consing bignums. 
-    ;; -- WHN 2000-10-02
-    *read-only-space-free-pointer*
-    *static-space-free-pointer*
-    *initial-dynamic-space-free-pointer*
+     ;; the floating point constants
+     *fp-constant-0d0*
+     *fp-constant-1d0*
+     *fp-constant-0f0*
+     *fp-constant-1f0*
+     ;; The following are all long-floats.
+     *fp-constant-0l0*
+     *fp-constant-1l0*
+     *fp-constant-pi*
+     *fp-constant-l2t*
+     *fp-constant-l2e*
+     *fp-constant-lg2*
+     *fp-constant-ln2*
 
-    ;; things needed for non-local exit
-    *current-catch-block*
-    *current-unwind-protect-block*
-    *alien-stack*
+     ;; For GC-AND-SAVE
+     *restart-lisp-function*
 
-    ;; interrupt handling
-    *pseudo-atomic-atomic*
-    *pseudo-atomic-interrupted*
-    sb!unix::*interrupts-enabled*
-    sb!unix::*interrupt-pending*
-    *free-interrupt-context-index*
+     ;; Needed for callbacks to work across saving cores. see
+     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
+     ;; details.
+     sb!alien::*enter-alien-callback*
 
-    *free-tls-index*
-    
-    *allocation-pointer*
-    *binding-stack-pointer*
-    *binding-stack-start*
-    *control-stack-start*
-    *control-stack-end*
-
-    *need-to-collect-garbage*
-
-    ;; the floating point constants
-    *fp-constant-0d0*
-    *fp-constant-1d0*
-    *fp-constant-0f0*
-    *fp-constant-1f0*
-    ;; The following are all long-floats.
-    *fp-constant-0l0*
-    *fp-constant-1l0*
-    *fp-constant-pi*
-    *fp-constant-l2t*
-    *fp-constant-l2e*
-    *fp-constant-lg2*
-    *fp-constant-ln2*
-
-    ;; The ..SLOT-UNBOUND.. symbol is static in order to optimise the
-    ;; common slot unbound check.
-    ;;
-    ;; FIXME: In SBCL, the CLOS code has become sufficiently tightly
-    ;; integrated into the system that it'd probably make sense to use
-    ;; the ordinary unbound marker for this.
-    sb!pcl::..slot-unbound..
-    ))
+     ;; see comments in ../x86-64/parms.lisp
+     sb!pcl::..slot-unbound..
+     )))
 
 (defparameter *static-funs*
   '(length
