@@ -30,58 +30,26 @@
 ;;; so we've left 'em in.)
 (when (eq *boot-state* 'complete)
   (error "Trying to load (or compile) PCL in an environment in which it~%~
-	  has already been loaded. This doesn't work, you will have to~%~
-	  get a fresh lisp (reboot) and then load PCL."))
+          has already been loaded. This doesn't work, you will have to~%~
+          get a fresh lisp (reboot) and then load PCL."))
 (when *boot-state*
   (cerror "Try loading (or compiling) PCL anyways."
-	  "Trying to load (or compile) PCL in an environment in which it~%~
-	   has already been partially loaded. This may not work, you may~%~
-	   need to get a fresh lisp (reboot) and then load PCL."))
+          "Trying to load (or compile) PCL in an environment in which it~%~
+           has already been partially loaded. This may not work, you may~%~
+           need to get a fresh lisp (reboot) and then load PCL."))
 
-;;; comments from CMU CL version of PCL:
-;;;     This is like fdefinition on the Lispm. If Common Lisp had
-;;;   something like function specs I wouldn't need this. On the other
-;;;   hand, I don't like the way this really works so maybe function
-;;;   specs aren't really right either?
-;;;     I also don't understand the real implications of a Lisp-1 on this
-;;;   sort of thing. Certainly some of the lossage in all of this is
-;;;   because these SPECs name global definitions.
-;;;     Note that this implementation is set up so that an implementation
-;;;   which has a 'real' function spec mechanism can use that instead
-;;;   and in that way get rid of setf generic function names.
-(defmacro parse-gspec (spec
-		       (non-setf-var . non-setf-case))
-  `(let ((,non-setf-var ,spec)) ,@non-setf-case))
-
-;;; If symbol names a function which is traced, return the untraced
-;;; definition. This lets us get at the generic function object even
-;;; when it is traced.
-(defun unencapsulated-fdefinition (symbol)
-  (fdefinition symbol))
-
-;;; If symbol names a function which is traced, redefine the `real'
-;;; definition without affecting the trace.
-(defun fdefine-carefully (name new-definition)
-  (progn
-    (sb-c::note-name-defined name :function)
-    new-definition)
-  (setf (fdefinition name) new-definition))
-
-(defun gboundp (spec)
-  (parse-gspec spec
-    (name (fboundp name))))
-
-(defun gmakunbound (spec)
-  (parse-gspec spec
-    (name (fmakunbound name))))
-
+#-sb-fluid (declaim (inline gdefinition))
 (defun gdefinition (spec)
-  (parse-gspec spec
-    (name (unencapsulated-fdefinition name))))
+  ;; This is null layer right now, but once FDEFINITION stops bypasssing
+  ;; fwrappers/encapsulations we can do that here.
+  (fdefinition spec))
 
 (defun (setf gdefinition) (new-value spec)
-  (parse-gspec spec
-    (name (fdefine-carefully name new-value))))
+  ;; This is almost a null layer right now, but once (SETF
+  ;; FDEFINITION) stops bypasssing fwrappers/encapsulations we can do
+  ;; that here.
+  (sb-c::note-name-defined spec :function) ; FIXME: do we need this? Why?
+  (setf (fdefinition spec) new-value))
 
 ;;;; type specifier hackery
 
@@ -89,7 +57,7 @@
 (defun coerce-to-class (class &optional make-forward-referenced-class-p)
   (if (symbolp class)
       (or (find-class class (not make-forward-referenced-class-p))
-	  (ensure-class class))
+          (ensure-class class))
       class))
 
 ;;; interface
@@ -97,49 +65,49 @@
   (when (consp type)
     (setq args (cdr type) type (car type)))
   (cond ((symbolp type)
-	 (or (and (null args) (find-class type))
-	     (ecase type
-	       (class    (coerce-to-class (car args)))
-	       (prototype (make-instance 'class-prototype-specializer
-					 :object (coerce-to-class (car args))))
-	       (class-eq (class-eq-specializer (coerce-to-class (car args))))
-	       (eql      (intern-eql-specializer (car args))))))
-	;; FIXME: do we still need this?
-	((and (null args) (typep type 'classoid))
-	 (or (classoid-pcl-class type)
-	     (ensure-non-standard-class (classoid-name type))))
-	((specializerp type) type)))
+         (or (and (null args) (find-class type))
+             (ecase type
+               (class    (coerce-to-class (car args)))
+               (prototype (make-instance 'class-prototype-specializer
+                                         :object (coerce-to-class (car args))))
+               (class-eq (class-eq-specializer (coerce-to-class (car args))))
+               (eql      (intern-eql-specializer (car args))))))
+        ;; FIXME: do we still need this?
+        ((and (null args) (typep type 'classoid))
+         (or (classoid-pcl-class type)
+             (ensure-non-standard-class (classoid-name type))))
+        ((specializerp type) type)))
 
 ;;; interface
 (defun type-from-specializer (specl)
   (cond ((eq specl t)
-	 t)
-	((consp specl)
-	 (unless (member (car specl) '(class prototype class-eq eql))
-	   (error "~S is not a legal specializer type." specl))
-	 specl)
-	((progn
-	   (when (symbolp specl)
-	     ;;maybe (or (find-class specl nil) (ensure-class specl)) instead?
-	     (setq specl (find-class specl)))
-	   (or (not (eq *boot-state* 'complete))
-	       (specializerp specl)))
-	 (specializer-type specl))
-	(t
-	 (error "~S is neither a type nor a specializer." specl))))
+         t)
+        ((consp specl)
+         (unless (member (car specl) '(class prototype class-eq eql))
+           (error "~S is not a legal specializer type." specl))
+         specl)
+        ((progn
+           (when (symbolp specl)
+             ;;maybe (or (find-class specl nil) (ensure-class specl)) instead?
+             (setq specl (find-class specl)))
+           (or (not (eq *boot-state* 'complete))
+               (specializerp specl)))
+         (specializer-type specl))
+        (t
+         (error "~S is neither a type nor a specializer." specl))))
 
 (defun type-class (type)
   (declare (special *the-class-t*))
   (setq type (type-from-specializer type))
   (if (atom type)
       (if (eq type t)
-	  *the-class-t*
-	  (error "bad argument to TYPE-CLASS"))
+          *the-class-t*
+          (error "bad argument to TYPE-CLASS"))
       (case (car type)
-	(eql (class-of (cadr type)))
-	(prototype (class-of (cadr type))) ;?
-	(class-eq (cadr type))
-	(class (cadr type)))))
+        (eql (class-of (cadr type)))
+        (prototype (class-of (cadr type))) ;?
+        (class-eq (cadr type))
+        (class (cadr type)))))
 
 (defun class-eq-type (class)
   (specializer-type (class-eq-specializer class)))
@@ -150,34 +118,34 @@
 ;;; class objects or types where they should.
 (defun *normalize-type (type)
   (cond ((consp type)
-	 (if (member (car type) '(not and or))
-	     `(,(car type) ,@(mapcar #'*normalize-type (cdr type)))
-	     (if (null (cdr type))
-		 (*normalize-type (car type))
-		 type)))
-	((symbolp type)
-	 (let ((class (find-class type nil)))
-	   (if class
-	       (let ((type (specializer-type class)))
-		 (if (listp type) type `(,type)))
-	       `(,type))))
-	((or (not (eq *boot-state* 'complete))
-	     (specializerp type))
-	 (specializer-type type))
-	(t
-	 (error "~S is not a type." type))))
+         (if (member (car type) '(not and or))
+             `(,(car type) ,@(mapcar #'*normalize-type (cdr type)))
+             (if (null (cdr type))
+                 (*normalize-type (car type))
+                 type)))
+        ((symbolp type)
+         (let ((class (find-class type nil)))
+           (if class
+               (let ((type (specializer-type class)))
+                 (if (listp type) type `(,type)))
+               `(,type))))
+        ((or (not (eq *boot-state* 'complete))
+             (specializerp type))
+         (specializer-type type))
+        (t
+         (error "~S is not a type." type))))
 
 ;;; internal to this file...
 (defun convert-to-system-type (type)
   (case (car type)
     ((not and or) `(,(car type) ,@(mapcar #'convert-to-system-type
-					  (cdr type))))
+                                          (cdr type))))
     ((class class-eq) ; class-eq is impossible to do right
      (layout-classoid (class-wrapper (cadr type))))
     (eql type)
     (t (if (null (cdr type))
-	   (car type)
-	   type))))
+           (car type)
+           type))))
 
 ;;; Writing the missing NOT and AND clauses will improve the quality
 ;;; of code generated by GENERATE-DISCRIMINATION-NET, but calling
@@ -185,31 +153,31 @@
 ;;; slow. *SUBTYPEP is used by PCL itself, and must be fast.
 ;;;
 ;;; FIXME: SB-KERNEL has fast-and-not-quite-precise type code for use
-;;; in the compiler. Could we share some of it here? 
+;;; in the compiler. Could we share some of it here?
 (defun *subtypep (type1 type2)
   (if (equal type1 type2)
       (values t t)
       (if (eq *boot-state* 'early)
-	  (values (eq type1 type2) t)
-	  (let ((*in-precompute-effective-methods-p* t))
-	    (declare (special *in-precompute-effective-methods-p*))
-	    ;; FIXME: *IN-PRECOMPUTE-EFFECTIVE-METHODS-P* is not a
-	    ;; good name. It changes the way
-	    ;; CLASS-APPLICABLE-USING-CLASS-P works.
-	    (setq type1 (*normalize-type type1))
-	    (setq type2 (*normalize-type type2))
-	    (case (car type2)
-	      (not
-	       (values nil nil)) ; XXX We should improve this.
-	      (and
-	       (values nil nil)) ; XXX We should improve this.
-	      ((eql wrapper-eq class-eq class)
-	       (multiple-value-bind (app-p maybe-app-p)
-		   (specializer-applicable-using-type-p type2 type1)
-		 (values app-p (or app-p (not maybe-app-p)))))
-	      (t
-	       (subtypep (convert-to-system-type type1)
-			 (convert-to-system-type type2))))))))
+          (values (eq type1 type2) t)
+          (let ((*in-precompute-effective-methods-p* t))
+            (declare (special *in-precompute-effective-methods-p*))
+            ;; FIXME: *IN-PRECOMPUTE-EFFECTIVE-METHODS-P* is not a
+            ;; good name. It changes the way
+            ;; CLASS-APPLICABLE-USING-CLASS-P works.
+            (setq type1 (*normalize-type type1))
+            (setq type2 (*normalize-type type2))
+            (case (car type2)
+              (not
+               (values nil nil)) ; XXX We should improve this.
+              (and
+               (values nil nil)) ; XXX We should improve this.
+              ((eql wrapper-eq class-eq class)
+               (multiple-value-bind (app-p maybe-app-p)
+                   (specializer-applicable-using-type-p type2 type1)
+                 (values app-p (or app-p (not maybe-app-p)))))
+              (t
+               (subtypep (convert-to-system-type type1)
+                         (convert-to-system-type type2))))))))
 
 (defvar *built-in-class-symbols* ())
 (defvar *built-in-wrapper-symbols* ())
@@ -217,14 +185,14 @@
 (defun get-built-in-class-symbol (class-name)
   (or (cadr (assq class-name *built-in-class-symbols*))
       (let ((symbol (make-class-symbol class-name)))
-	(push (list class-name symbol) *built-in-class-symbols*)
-	symbol)))
+        (push (list class-name symbol) *built-in-class-symbols*)
+        symbol)))
 
 (defun get-built-in-wrapper-symbol (class-name)
   (or (cadr (assq class-name *built-in-wrapper-symbols*))
       (let ((symbol (make-wrapper-symbol class-name)))
-	(push (list class-name symbol) *built-in-wrapper-symbols*)
-	symbol)))
+        (push (list class-name symbol) *built-in-wrapper-symbols*)
+        symbol)))
 
 (pushnew '%class *var-declarations*)
 (pushnew '%variable-rebinding *var-declarations*)
@@ -232,13 +200,8 @@
 (defun variable-class (var env)
   (caddr (var-declaration 'class var env)))
 
-(defvar *name->class->slotd-table* (make-hash-table))
-
 (defvar *standard-method-combination*)
 
-(defun make-class-predicate-name (name)
-  (list 'class-predicate name))
-  
 (defun plist-value (object name)
   (getf (object-plist object) name))
 
@@ -246,8 +209,8 @@
   (if new-value
       (setf (getf (object-plist object) name) new-value)
       (progn
-	(remf (object-plist object) name)
-	nil)))
+        (remf (object-plist object) name)
+        nil)))
 
 ;;;; built-in classes
 
@@ -256,59 +219,57 @@
 (/show "about to set up SB-PCL::*BUILT-IN-CLASSES*")
 (defvar *built-in-classes*
   (labels ((direct-supers (class)
-	     (/noshow "entering DIRECT-SUPERS" (classoid-name class))
-	     (if (typep class 'built-in-classoid)
-		 (built-in-classoid-direct-superclasses class)
-		 (let ((inherits (layout-inherits
-				  (classoid-layout class))))
-		   (/noshow inherits)
-		   (list (svref inherits (1- (length inherits)))))))
-	   (direct-subs (class)
-	     (/noshow "entering DIRECT-SUBS" (classoid-name class))
-	     (collect ((res))
-	       (let ((subs (classoid-subclasses class)))
-		 (/noshow subs)
-		 (when subs
-		   (dohash (sub v subs)
-		     (declare (ignore v))
-		     (/noshow sub)
-		     (when (member class (direct-supers sub))
-		       (res sub)))))
-	       (res))))
+             (/noshow "entering DIRECT-SUPERS" (classoid-name class))
+             (if (typep class 'built-in-classoid)
+                 (built-in-classoid-direct-superclasses class)
+                 (let ((inherits (layout-inherits
+                                  (classoid-layout class))))
+                   (/noshow inherits)
+                   (list (svref inherits (1- (length inherits)))))))
+           (direct-subs (class)
+             (/noshow "entering DIRECT-SUBS" (classoid-name class))
+             (collect ((res))
+               (let ((subs (classoid-subclasses class)))
+                 (/noshow subs)
+                 (when subs
+                   (dohash (sub v subs)
+                     (declare (ignore v))
+                     (/noshow sub)
+                     (when (member class (direct-supers sub))
+                       (res sub)))))
+               (res))))
     (mapcar (lambda (kernel-bic-entry)
-	      (/noshow "setting up" kernel-bic-entry)
-	      (let* ((name (car kernel-bic-entry))
-		     (class (find-classoid name))
-		     (prototype-form
-		      (getf (cdr kernel-bic-entry) :prototype-form)))
-		(/noshow name class)
-		`(,name
-		  ,(mapcar #'classoid-name (direct-supers class))
-		  ,(mapcar #'classoid-name (direct-subs class))
-		  ,(map 'list
-			(lambda (x)
-			  (classoid-name
-			   (layout-classoid x)))
-			(reverse
-			 (layout-inherits
-			  (classoid-layout class))))
-		  ,(if prototype-form
-		       (eval prototype-form)
-		       ;; This is the default prototype value which
-		       ;; was used, without explanation, by the CMU CL
-		       ;; code we're derived from. Evidently it's safe
-		       ;; in all relevant cases.
-		       42))))
-	    (remove-if (lambda (kernel-bic-entry)
-			 (member (first kernel-bic-entry)
-				 ;; I'm not sure why these are removed from
-				 ;; the list, but that's what the original
-				 ;; CMU CL code did. -- WHN 20000715
-				 '(t instance
-				     funcallable-instance
-				     function stream 
-				     file-stream string-stream)))
-		       sb-kernel::*built-in-classes*))))
+              (/noshow "setting up" kernel-bic-entry)
+              (let* ((name (car kernel-bic-entry))
+                     (class (find-classoid name))
+                     (prototype-form
+                      (getf (cdr kernel-bic-entry) :prototype-form)))
+                (/noshow name class)
+                `(,name
+                  ,(mapcar #'classoid-name (direct-supers class))
+                  ,(mapcar #'classoid-name (direct-subs class))
+                  ,(map 'list
+                        (lambda (x)
+                          (classoid-name
+                           (layout-classoid x)))
+                        (reverse
+                         (layout-inherits
+                          (classoid-layout class))))
+                  ,(if prototype-form
+                       (eval prototype-form)
+                       ;; This is the default prototype value which
+                       ;; was used, without explanation, by the CMU CL
+                       ;; code we're derived from. Evidently it's safe
+                       ;; in all relevant cases.
+                       42))))
+            (remove-if (lambda (kernel-bic-entry)
+                         (member (first kernel-bic-entry)
+                                 ;; I'm not sure why these are removed from
+                                 ;; the list, but that's what the original
+                                 ;; CMU CL code did. -- WHN 20000715
+                                 '(t function stream
+                                     file-stream string-stream)))
+                       sb-kernel::*built-in-classes*))))
 (/noshow "done setting up SB-PCL::*BUILT-IN-CLASSES*")
 
 ;;;; the classes that define the kernel of the metabraid
@@ -316,16 +277,10 @@
 (defclass t () ()
   (:metaclass built-in-class))
 
-(defclass instance (t) ()
-  (:metaclass built-in-class))
-
 (defclass function (t) ()
   (:metaclass built-in-class))
 
-(defclass funcallable-instance (function) ()
-  (:metaclass built-in-class))
-
-(defclass stream (instance) ()
+(defclass stream (t) ()
   (:metaclass built-in-class))
 
 (defclass file-stream (stream) ()
@@ -337,57 +292,292 @@
 (defclass slot-object (t) ()
   (:metaclass slot-class))
 
-(defclass condition (slot-object instance) ()
+(defclass condition (slot-object) ()
   (:metaclass condition-class))
 
-(defclass structure-object (slot-object instance) ()
+(defclass structure-object (slot-object) ()
   (:metaclass structure-class))
 
 (defstruct (dead-beef-structure-object
-	    (:constructor |STRUCTURE-OBJECT class constructor|)
-	    (:copier nil)))
+            (:constructor |STRUCTURE-OBJECT class constructor|)
+            (:copier nil)))
 
-(defclass std-object (slot-object) ()
-  (:metaclass std-class))
+(defclass standard-object (slot-object) ())
 
-(defclass standard-object (std-object instance) ())
-
-(defclass funcallable-standard-object (std-object funcallable-instance)
+(defclass funcallable-standard-object (standard-object function)
   ()
   (:metaclass funcallable-standard-class))
 
-(defclass specializer (standard-object)
-  ((type
-    :initform nil
-    :reader specializer-type)))
+(defclass metaobject (standard-object) ())
 
-(defclass definition-source-mixin (std-object)
-  ((source
-    :initform *load-pathname*
-    :reader definition-source
-    :initarg :definition-source))
-  (:metaclass std-class))
+(defclass generic-function (dependent-update-mixin
+                            definition-source-mixin
+                            metaobject
+                            funcallable-standard-object)
+  ((%documentation :initform nil :initarg :documentation)
+   ;; We need to make a distinction between the methods initially set
+   ;; up by :METHOD options to DEFGENERIC and the ones set up later by
+   ;; DEFMETHOD, because ANSI specifies that executing DEFGENERIC on
+   ;; an already-DEFGENERICed function clears the methods set by the
+   ;; previous DEFGENERIC, but not methods set by DEFMETHOD. (Making
+   ;; this distinction seems a little kludgy, but it has the positive
+   ;; effect of making it so that loading a file a.lisp containing
+   ;; DEFGENERIC, then loading a second file b.lisp containing
+   ;; DEFMETHOD, then modifying and reloading a.lisp and/or b.lisp
+   ;; tends to leave the generic function in a state consistent with
+   ;; the most-recently-loaded state of a.lisp and b.lisp.)
+   (initial-methods :initform ()
+                    :accessor generic-function-initial-methods))
+  (:metaclass funcallable-standard-class))
 
-(defclass plist-mixin (std-object)
-  ((plist
-    :initform ()
-    :accessor object-plist))
-  (:metaclass std-class))
-
-(defclass dependent-update-mixin (plist-mixin)
-  ()
-  (:metaclass std-class))
-
-;;; The class CLASS is a specified basic class. It is the common
-;;; superclass of any kind of class. That is, any class that can be a
-;;; metaclass must have the class CLASS in its class precedence list.
-(defclass class (dependent-update-mixin
-		 definition-source-mixin
-		 specializer)
+(defclass standard-generic-function (generic-function)
   ((name
     :initform nil
-    :initarg  :name
-    :accessor class-name)
+    :initarg :name
+    :reader generic-function-name)
+   (methods
+    :initform ()
+    :accessor generic-function-methods
+    :type list)
+   (method-class
+    :initarg :method-class
+    :accessor generic-function-method-class)
+   (%method-combination
+    :initarg :method-combination
+    :accessor generic-function-method-combination)
+   (declarations
+    ;; KLUDGE: AMOP specifies :DECLARATIONS, while ANSI specifies
+    ;; :DECLARE.  Allow either (but FIXME: maybe a note or a warning
+    ;; might be appropriate).
+    :initarg :declarations
+    :initarg :declare
+    :initform ()
+    :accessor generic-function-declarations)
+   (arg-info
+    :initform (make-arg-info)
+    :reader gf-arg-info)
+   (dfun-state
+    :initform ()
+    :accessor gf-dfun-state))
+  (:metaclass funcallable-standard-class)
+  (:default-initargs :method-class *the-class-standard-method*
+                     :method-combination *standard-method-combination*))
+
+(defclass method (metaobject) ())
+
+(defclass standard-method (definition-source-mixin plist-mixin method)
+  ((%generic-function
+    :initform nil
+    :accessor method-generic-function)
+   #+nil ; implemented by PLIST
+   (qualifiers
+    :initform ()
+    :initarg  :qualifiers
+    :reader method-qualifiers)
+   (specializers
+    :initform ()
+    :initarg  :specializers
+    :reader method-specializers)
+   (lambda-list
+    :initform ()
+    :initarg  :lambda-list
+    :reader method-lambda-list)
+   (%function :initform nil :initarg :function)
+   (fast-function
+    :initform nil
+    :initarg :fast-function             ;no writer
+    :reader method-fast-function)
+   (%documentation :initform nil :initarg :documentation)))
+
+(defclass standard-accessor-method (standard-method)
+  ((slot-name :initform nil :initarg :slot-name
+              :reader accessor-method-slot-name)
+   (%slot-definition :initform nil :initarg :slot-definition
+                     :reader accessor-method-slot-definition)))
+
+(defclass standard-reader-method (standard-accessor-method) ())
+(defclass standard-writer-method (standard-accessor-method) ())
+;;; an extension, apparently.
+(defclass standard-boundp-method (standard-accessor-method) ())
+
+(defclass method-combination (metaobject)
+  ((%documentation :initform nil :initarg :documentation)))
+
+(defclass standard-method-combination (definition-source-mixin
+                                       method-combination)
+  ((type-name
+    :reader method-combination-type-name
+    :initarg :type-name)
+   (options
+    :reader method-combination-options
+    :initarg :options)))
+
+(defclass long-method-combination (standard-method-combination)
+  ((function
+    :initarg :function
+    :reader long-method-combination-function)
+   (args-lambda-list
+    :initarg :args-lambda-list
+    :reader long-method-combination-args-lambda-list)))
+
+(defclass short-method-combination (standard-method-combination)
+  ((operator
+    :reader short-combination-operator
+    :initarg :operator)
+   (identity-with-one-argument
+    :reader short-combination-identity-with-one-argument
+    :initarg :identity-with-one-argument)))
+
+(defclass slot-definition (metaobject)
+  ((name
+    :initform nil
+    :initarg :name
+    :accessor slot-definition-name)
+   (initform
+    :initform nil
+    :initarg :initform
+    :accessor slot-definition-initform)
+   (initfunction
+    :initform nil
+    :initarg :initfunction
+    :accessor slot-definition-initfunction)
+   (readers
+    :initform nil
+    :initarg :readers
+    :accessor slot-definition-readers)
+   (writers
+    :initform nil
+    :initarg :writers
+    :accessor slot-definition-writers)
+   (initargs
+    :initform nil
+    :initarg :initargs
+    :accessor slot-definition-initargs)
+   (%type :initform t :initarg :type :accessor slot-definition-type)
+   (%documentation
+    :initform nil :initarg :documentation
+    ;; KLUDGE: we need a reader for bootstrapping purposes, in
+    ;; COMPUTE-EFFECTIVE-SLOT-DEFINITION-INITARGS.
+    :reader %slot-definition-documentation)
+   (%class :initform nil :initarg :class :accessor slot-definition-class)))
+
+(defclass standard-slot-definition (slot-definition)
+  ((allocation
+    :initform :instance
+    :initarg :allocation
+    :accessor slot-definition-allocation)
+   (allocation-class
+    :initform nil
+    :initarg :allocation-class
+    :accessor slot-definition-allocation-class)))
+
+(defclass condition-slot-definition (slot-definition)
+  ((allocation
+    :initform :instance
+    :initarg :allocation
+    :accessor slot-definition-allocation)
+   (allocation-class
+    :initform nil
+    :initarg :allocation-class
+    :accessor slot-definition-allocation-class)))
+
+(defclass structure-slot-definition (slot-definition)
+  ((defstruct-accessor-symbol
+     :initform nil
+     :initarg :defstruct-accessor-symbol
+     :accessor slot-definition-defstruct-accessor-symbol)
+   (internal-reader-function
+     :initform nil
+     :initarg :internal-reader-function
+     :accessor slot-definition-internal-reader-function)
+   (internal-writer-function
+     :initform nil
+     :initarg :internal-writer-function
+     :accessor slot-definition-internal-writer-function)))
+
+(defclass direct-slot-definition (slot-definition)
+  ())
+
+(defclass effective-slot-definition (slot-definition)
+  ((reader-function ; (lambda (object) ...)
+    :accessor slot-definition-reader-function)
+   (writer-function ; (lambda (new-value object) ...)
+    :accessor slot-definition-writer-function)
+   (boundp-function ; (lambda (object) ...)
+    :accessor slot-definition-boundp-function)
+   (accessor-flags
+    :initform 0)))
+
+(defclass standard-direct-slot-definition (standard-slot-definition
+                                           direct-slot-definition)
+  ())
+
+(defclass standard-effective-slot-definition (standard-slot-definition
+                                              effective-slot-definition)
+  ((location ; nil, a fixnum, a cons: (slot-name . value)
+    :initform nil
+    :accessor slot-definition-location)))
+
+(defclass condition-direct-slot-definition (condition-slot-definition
+                                            direct-slot-definition)
+  ())
+
+(defclass condition-effective-slot-definition (condition-slot-definition
+                                               effective-slot-definition)
+  ())
+
+(defclass structure-direct-slot-definition (structure-slot-definition
+                                            direct-slot-definition)
+  ())
+
+(defclass structure-effective-slot-definition (structure-slot-definition
+                                               effective-slot-definition)
+  ())
+
+(defclass specializer (metaobject)
+  ;; KLUDGE: in sbcl-0.9.10.2 this was renamed from TYPE, which was an
+  ;; external symbol of the CL package and hence potentially collides
+  ;; with user code.  Renaming this to %TYPE, however, is the coward's
+  ;; way out, because the objects that PCL puts in this slot aren't
+  ;; (quite) types: they are closer to kinds of specializer.  However,
+  ;; the wholesale renaming and disentangling of specializers didn't
+  ;; appeal.  (See also message <sqd5hrclb2.fsf@cam.ac.uk> and
+  ;; responses in comp.lang.lisp).  -- CSR, 2006-02-27
+  ((%type :initform nil :reader specializer-type)))
+
+(defclass specializer-with-object (specializer) ())
+
+(defclass exact-class-specializer (specializer) ())
+
+(defclass class-eq-specializer (exact-class-specializer
+                                specializer-with-object)
+  ((object :initarg :class
+           :reader specializer-class
+           :reader specializer-object)))
+
+(defclass class-prototype-specializer (specializer-with-object)
+  ((object :initarg :class
+           :reader specializer-class
+           :reader specializer-object)))
+
+(defclass eql-specializer (exact-class-specializer specializer-with-object)
+  ((object :initarg :object :reader specializer-object
+           :reader eql-specializer-object)))
+
+(defvar *eql-specializer-table* (make-hash-table :test 'eql))
+
+(defun intern-eql-specializer (object)
+  (or (gethash object *eql-specializer-table*)
+      (setf (gethash object *eql-specializer-table*)
+            (make-instance 'eql-specializer :object object))))
+
+(defclass class (dependent-update-mixin
+                 definition-source-mixin
+                 specializer)
+  ((name
+    :initform nil
+    :initarg :name
+    :reader class-name)
    (class-eq-specializer
     :initform nil
     :reader class-eq-specializer)
@@ -402,10 +592,7 @@
     :reader class-direct-subclasses)
    (direct-methods
     :initform (cons nil nil))
-   (predicate-name
-    :initform nil
-    :reader class-predicate-name)
-   (documentation
+   (%documentation
     :initform nil
     :initarg :documentation)
    (finalized-p
@@ -419,13 +606,13 @@
   (let ((name (class-name class)))
     (unless (and name (eq (find-class name nil) class))
       (error "~@<Can't use anonymous or undefined class as constant: ~S~:@>"
-	     class))
+             class))
     `(find-class ',name)))
 
 ;;; The class PCL-CLASS is an implementation-specific common
 ;;; superclass of all specified subclasses of the class CLASS.
 (defclass pcl-class (class)
-  ((class-precedence-list
+  ((%class-precedence-list
     :reader class-precedence-list)
    ;; KLUDGE: see note in CPL-OR-NIL
    (cpl-available-p
@@ -481,264 +668,16 @@
     :initform nil
     :initarg :from-defclass-p)))
 
-(defclass specializer-with-object (specializer) ())
-
-(defclass exact-class-specializer (specializer) ())
-
-(defclass class-eq-specializer (exact-class-specializer
-				specializer-with-object)
-  ((object :initarg :class
-	   :reader specializer-class
-	   :reader specializer-object)))
-
-(defclass class-prototype-specializer (specializer-with-object)
-  ((object :initarg :class
-	   :reader specializer-class
-	   :reader specializer-object)))
-
-(defclass eql-specializer (exact-class-specializer specializer-with-object)
-  ((object :initarg :object :reader specializer-object
-	   :reader eql-specializer-object)))
-
-(defvar *eql-specializer-table* (make-hash-table :test 'eql))
-
-(defun intern-eql-specializer (object)
-  (or (gethash object *eql-specializer-table*)
-      (setf (gethash object *eql-specializer-table*)
-	    (make-instance 'eql-specializer :object object))))
-
-;;;; slot definitions
-
-(defclass slot-definition (standard-object)
-  ((name
+(defclass definition-source-mixin (standard-object)
+  ((source
     :initform nil
-    :initarg :name
-    :accessor slot-definition-name)
-   (initform
-    :initform nil
-    :initarg :initform
-    :accessor slot-definition-initform)
-   (initfunction
-    :initform nil
-    :initarg :initfunction
-    :accessor slot-definition-initfunction)
-   (readers
-    :initform nil
-    :initarg :readers
-    :accessor slot-definition-readers)
-   (writers
-    :initform nil
-    :initarg :writers
-    :accessor slot-definition-writers)
-   (initargs
-    :initform nil
-    :initarg :initargs
-    :accessor slot-definition-initargs)
-   (type
-    :initform t
-    :initarg :type
-    :accessor slot-definition-type)
-   (documentation
-    :initform nil
-    :initarg :documentation)
-   (class
-    :initform nil
-    :initarg :class
-    :accessor slot-definition-class)))
+    :reader definition-source
+    :initarg :definition-source)))
 
-(defclass standard-slot-definition (slot-definition)
-  ((allocation
-    :initform :instance
-    :initarg :allocation
-    :accessor slot-definition-allocation)
-   (allocation-class
-    :initform nil
-    :initarg :allocation-class
-    :accessor slot-definition-allocation-class)))
+(defclass plist-mixin (standard-object)
+  ((plist :initform () :accessor object-plist)))
 
-(defclass condition-slot-definition (slot-definition)
-  ((allocation
-    :initform :instance
-    :initarg :allocation
-    :accessor slot-definition-allocation)
-   (allocation-class
-    :initform nil
-    :initarg :allocation-class
-    :accessor slot-definition-allocation-class)))
-
-(defclass structure-slot-definition (slot-definition)
-  ((defstruct-accessor-symbol
-     :initform nil
-     :initarg :defstruct-accessor-symbol
-     :accessor slot-definition-defstruct-accessor-symbol)
-   (internal-reader-function
-     :initform nil
-     :initarg :internal-reader-function
-     :accessor slot-definition-internal-reader-function)
-   (internal-writer-function
-     :initform nil
-     :initarg :internal-writer-function
-     :accessor slot-definition-internal-writer-function)))
-
-(defclass direct-slot-definition (slot-definition)
-  ())
-
-(defclass effective-slot-definition (slot-definition)
-  ((reader-function ; (lambda (object) ...)
-    :accessor slot-definition-reader-function)
-   (writer-function ; (lambda (new-value object) ...)
-    :accessor slot-definition-writer-function)
-   (boundp-function ; (lambda (object) ...)
-    :accessor slot-definition-boundp-function)
-   (accessor-flags
-    :initform 0)))
-
-(defclass standard-direct-slot-definition (standard-slot-definition
-					   direct-slot-definition)
-  ())
-
-(defclass standard-effective-slot-definition (standard-slot-definition
-					      effective-slot-definition)
-  ((location ; nil, a fixnum, a cons: (slot-name . value)
-    :initform nil
-    :accessor slot-definition-location)))
-
-(defclass condition-direct-slot-definition (condition-slot-definition
-					    direct-slot-definition)
-  ())
-
-(defclass condition-effective-slot-definition (condition-slot-definition
-					       effective-slot-definition)
-  ())
-
-(defclass structure-direct-slot-definition (structure-slot-definition
-					    direct-slot-definition)
-  ())
-
-(defclass structure-effective-slot-definition (structure-slot-definition
-					       effective-slot-definition)
-  ())
-
-(defclass method (standard-object) ())
-
-(defclass standard-method (definition-source-mixin plist-mixin method)
-  ((generic-function
-    :initform nil	
-    :accessor method-generic-function)
-;;;     (qualifiers
-;;;	:initform ()
-;;;	:initarg  :qualifiers
-;;;	:reader method-qualifiers)
-   (specializers
-    :initform ()
-    :initarg  :specializers
-    :reader method-specializers)
-   (lambda-list
-    :initform ()
-    :initarg  :lambda-list
-    :reader method-lambda-list)
-   (function
-    :initform nil
-    :initarg :function)			;no writer
-   (fast-function
-    :initform nil
-    :initarg :fast-function		;no writer
-    :reader method-fast-function)
-   (documentation
-    :initform nil
-    :initarg :documentation)))
-
-(defclass standard-accessor-method (standard-method)
-  ((slot-name :initform nil
-	      :initarg :slot-name
-	      :reader accessor-method-slot-name)
-   (slot-definition :initform nil
-		    :initarg :slot-definition
-		    :reader accessor-method-slot-definition)))
-
-(defclass standard-reader-method (standard-accessor-method) ())
-
-(defclass standard-writer-method (standard-accessor-method) ())
-
-(defclass standard-boundp-method (standard-accessor-method) ())
-
-(defclass generic-function (dependent-update-mixin
-			    definition-source-mixin
-			    funcallable-standard-object)
-  ((documentation
-    :initform nil
-    :initarg :documentation)
-   ;; We need to make a distinction between the methods initially set
-   ;; up by :METHOD options to DEFGENERIC and the ones set up later by
-   ;; DEFMETHOD, because ANSI specifies that executing DEFGENERIC on
-   ;; an already-DEFGENERICed function clears the methods set by the
-   ;; previous DEFGENERIC, but not methods set by DEFMETHOD. (Making
-   ;; this distinction seems a little kludgy, but it has the positive
-   ;; effect of making it so that loading a file a.lisp containing
-   ;; DEFGENERIC, then loading a second file b.lisp containing
-   ;; DEFMETHOD, then modifying and reloading a.lisp and/or b.lisp
-   ;; tends to leave the generic function in a state consistent with
-   ;; the most-recently-loaded state of a.lisp and b.lisp.)
-   (initial-methods
-    :initform ()
-    :accessor generic-function-initial-methods))
-  (:metaclass funcallable-standard-class))
-
-(defclass standard-generic-function (generic-function)
-  ((name
-    :initform nil
-    :initarg :name
-    :accessor generic-function-name)
-   (methods
-    :initform ()
-    :accessor generic-function-methods
-    :type list)
-   (method-class
-    :initarg :method-class
-    :accessor generic-function-method-class)
-   (method-combination
-    :initarg :method-combination
-    :accessor generic-function-method-combination)
-   (declarations
-    ;; KLUDGE: AMOP specifies :DECLARATIONS, while ANSI specifies
-    ;; :DECLARE.  Allow either (but FIXME: maybe a note or a warning
-    ;; might be appropriate).
-    :initarg :declarations
-    :initarg :declare
-    :initform ()
-    :accessor generic-function-declarations)
-   (arg-info
-    :initform (make-arg-info)
-    :reader gf-arg-info)
-   (dfun-state
-    :initform ()
-    :accessor gf-dfun-state))
-  (:metaclass funcallable-standard-class)
-  (:default-initargs :method-class *the-class-standard-method*
-		     :method-combination *standard-method-combination*))
-
-(defclass method-combination (standard-object)
-  ((documentation
-    :reader method-combination-documentation
-    :initform nil
-    :initarg :documentation)))
-
-(defclass standard-method-combination (definition-source-mixin
-				       method-combination)
-  ((type
-    :reader method-combination-type
-    :initarg :type)
-   (options
-    :reader method-combination-options
-    :initarg :options)))
-
-(defclass long-method-combination (standard-method-combination)
-  ((function
-    :initarg :function
-    :reader long-method-combination-function)
-   (args-lambda-list
-    :initarg :args-lambda-list
-    :reader long-method-combination-args-lambda-list)))
+(defclass dependent-update-mixin (plist-mixin) ())
 
 (defparameter *early-class-predicates*
   '((specializer specializerp)
@@ -762,5 +701,5 @@
     (generic-function generic-function-p)
     (standard-generic-function standard-generic-function-p)
     (method-combination method-combination-p)
-    (long-method-combination long-method-combination-p)))
-
+    (long-method-combination long-method-combination-p)
+    (short-method-combination short-method-combination-p)))
