@@ -14,12 +14,12 @@
 ;;; functions, macros, and special forms
 (defmethod documentation ((x function) (doc-type (eql 't)))
   (if (typep x 'generic-function)
-      (slot-value x 'documentation)
+      (slot-value x '%documentation)
       (%fun-doc x)))
 
 (defmethod documentation ((x function) (doc-type (eql 'function)))
   (if (typep x 'generic-function)
-      (slot-value x 'documentation)
+      (slot-value x '%documentation)
       (%fun-doc x)))
 
 (defmethod documentation ((x list) (doc-type (eql 'function)))
@@ -43,19 +43,19 @@
 
 (defmethod (setf documentation) (new-value (x function) (doc-type (eql 't)))
   (if (typep x 'generic-function)
-      (setf (slot-value x 'documentation) new-value)
+      (setf (slot-value x '%documentation) new-value)
       (let ((name (%fun-name x)))
-	(when (and name (typep name '(or symbol cons)))
-	  (setf (info :function :documentation name) new-value))))
+        (when (and name (typep name '(or symbol cons)))
+          (setf (info :function :documentation name) new-value))))
   new-value)
 
 (defmethod (setf documentation)
     (new-value (x function) (doc-type (eql 'function)))
   (if (typep x 'generic-function)
-      (setf (slot-value x 'documentation) new-value)
+      (setf (slot-value x '%documentation) new-value)
       (let ((name (%fun-name x)))
-	(when (and name (typep name '(or symbol cons)))
-	  (setf (info :function :documentation name) new-value))))
+        (when (and name (typep name '(or symbol cons)))
+          (setf (info :function :documentation name) new-value))))
   new-value)
 
 (defmethod (setf documentation) (new-value (x list) (doc-type (eql 'function)))
@@ -66,8 +66,8 @@
   (setf (random-documentation x 'compiler-macro) new-value))
 
 (defmethod (setf documentation) (new-value
-				 (x symbol)
-				 (doc-type (eql 'function)))
+                                 (x symbol)
+                                 (doc-type (eql 'function)))
   (setf (info :function :documentation x) new-value))
 
 (defmethod (setf documentation)
@@ -79,22 +79,22 @@
 
 ;;; method combinations
 (defmethod documentation ((x method-combination) (doc-type (eql 't)))
-  (slot-value x 'documentation))
+  (slot-value x '%documentation))
 
 (defmethod documentation
     ((x method-combination) (doc-type (eql 'method-combination)))
-  (slot-value x 'documentation))
+  (slot-value x '%documentation))
 
 (defmethod documentation ((x symbol) (doc-type (eql 'method-combination)))
   (random-documentation x 'method-combination))
 
 (defmethod (setf documentation)
     (new-value (x method-combination) (doc-type (eql 't)))
-  (setf (slot-value x 'documentation) new-value))
+  (setf (slot-value x '%documentation) new-value))
 
 (defmethod (setf documentation)
     (new-value (x method-combination) (doc-type (eql 'method-combination)))
-  (setf (slot-value x 'documentation) new-value))
+  (setf (slot-value x '%documentation) new-value))
 
 (defmethod (setf documentation)
     (new-value (x symbol) (doc-type (eql 'method-combination)))
@@ -102,11 +102,11 @@
 
 ;;; methods
 (defmethod documentation ((x standard-method) (doc-type (eql 't)))
-  (slot-value x 'documentation))
+  (slot-value x '%documentation))
 
 (defmethod (setf documentation)
     (new-value (x standard-method) (doc-type (eql 't)))
-  (setf (slot-value x 'documentation) new-value))
+  (setf (slot-value x '%documentation) new-value))
 
 ;;; packages
 
@@ -132,78 +132,97 @@
   (values (info :type :documentation (class-name x))))
 
 (defmethod documentation ((x standard-class) (doc-type (eql 't)))
-  (slot-value x 'documentation))
+  (slot-value x '%documentation))
 
 (defmethod documentation ((x standard-class) (doc-type (eql 'type)))
-  (slot-value x 'documentation))
+  (slot-value x '%documentation))
+
+;;; although the CLHS doesn't mention this, it is reasonable to assume
+;;; that parallel treatment of condition-class was intended (if
+;;; condition-class is in fact not implemented as a standard-class or
+;;; structure-class).
+(defmethod documentation ((x condition-class) (doc-type (eql 't)))
+  (values (info :type :documentation (class-name x))))
+
+(defmethod documentation ((x condition-class) (doc-type (eql 'type)))
+  (values (info :type :documentation (class-name x))))
 
 (defmethod documentation ((x symbol) (doc-type (eql 'type)))
   (or (values (info :type :documentation x))
       (let ((class (find-class x nil)))
-	(when class
-	  (slot-value class 'documentation)))))
+        (when class
+          (slot-value class '%documentation)))))
 
 (defmethod documentation ((x symbol) (doc-type (eql 'structure)))
-  (cond ((eq (info :type :kind x) :instance)
-	 (values (info :type :documentation x)))
-	((info :typed-structure :info x)
-	 (values (info :typed-structure :documentation x)))
-	(t
-	 (error "~S is not the name of a structure type." x))))
+  (cond
+    ((structure-type-p x)
+     (values (info :type :documentation x)))
+    ((info :typed-structure :info x)
+     (values (info :typed-structure :documentation x)))
+    (t nil)))
 
 (defmethod (setf documentation) (new-value
-				 (x structure-class)
-				 (doc-type (eql 't)))
+                                 (x structure-class)
+                                 (doc-type (eql 't)))
   (setf (info :type :documentation (class-name x)) new-value))
 
 (defmethod (setf documentation) (new-value
-				 (x structure-class)
-				 (doc-type (eql 'type)))
+                                 (x structure-class)
+                                 (doc-type (eql 'type)))
   (setf (info :type :documentation (class-name x)) new-value))
 
 (defmethod (setf documentation) (new-value
-				 (x standard-class)
-				 (doc-type (eql 't)))
-  (setf (slot-value x 'documentation) new-value))
+                                 (x standard-class)
+                                 (doc-type (eql 't)))
+  (setf (slot-value x '%documentation) new-value))
 
 (defmethod (setf documentation) (new-value
-				 (x standard-class)
-				 (doc-type (eql 'type)))
-  (setf (slot-value x 'documentation) new-value))
+                                 (x standard-class)
+                                 (doc-type (eql 'type)))
+  (setf (slot-value x '%documentation) new-value))
+
+(defmethod (setf documentation) (new-value
+                                 (x condition-class)
+                                 (doc-type (eql 't)))
+  (setf (info :type :documentation (class-name x)) new-value))
+
+(defmethod (setf documentation) (new-value
+                                 (x condition-class)
+                                 (doc-type (eql 'type)))
+  (setf (info :type :documentation (class-name x)) new-value))
 
 (defmethod (setf documentation) (new-value (x symbol) (doc-type (eql 'type)))
   (if (or (structure-type-p x) (condition-type-p x))
       (setf (info :type :documentation x) new-value)
       (let ((class (find-class x nil)))
-	(if class
-	    (setf (slot-value class 'documentation) new-value)
-	    (setf (info :type :documentation x) new-value)))))
+        (if class
+            (setf (slot-value class '%documentation) new-value)
+            (setf (info :type :documentation x) new-value)))))
 
 (defmethod (setf documentation) (new-value
-				 (x symbol)
-				 (doc-type (eql 'structure)))
-  (cond ((eq (info :type :kind x) :instance)
-	 (setf (info :type :documentation x) new-value))
-	((info :typed-structure :info x)
-	 (setf (info :typed-structure :documentation x) new-value))
-	(t
-	 (error "~S is not the name of a structure type." x))))
-  
+                                 (x symbol)
+                                 (doc-type (eql 'structure)))
+  (cond
+    ((structure-type-p x)
+     (setf (info :type :documentation x) new-value))
+    ((info :typed-structure :info x)
+     (setf (info :typed-structure :documentation x) new-value))
+    (t new-value)))
 
 ;;; variables
 (defmethod documentation ((x symbol) (doc-type (eql 'variable)))
   (values (info :variable :documentation x)))
 
 (defmethod (setf documentation) (new-value
-				 (x symbol)
-				 (doc-type (eql 'variable)))
+                                 (x symbol)
+                                 (doc-type (eql 'variable)))
   (setf (info :variable :documentation x) new-value))
 
 ;;; default if DOC-TYPE doesn't match one of the specified types
 (defmethod documentation (object doc-type)
   (warn "unsupported DOCUMENTATION: type ~S for object ~S"
-	doc-type
-	(type-of object))
+        doc-type
+        (type-of object))
   nil)
 
 ;;; default if DOC-TYPE doesn't match one of the specified types
@@ -212,19 +231,19 @@
   ;; doc types an implementation is permitted to discard docs at any time
   ;; for any reason, this feels to me more like a warning. -- WHN 19991214
   (warn "discarding unsupported DOCUMENTATION of type ~S for object ~S"
-	doc-type
-	(type-of object))
+        doc-type
+        (type-of object))
   new-value)
 
 ;;; extra-standard methods, for getting at slot documentation
 (defmethod documentation ((slotd standard-slot-definition) (doc-type (eql 't)))
   (declare (ignore doc-type))
-  (slot-value slotd 'documentation))
+  (slot-value slotd '%documentation))
 
 (defmethod (setf documentation)
     (new-value (slotd standard-slot-definition) (doc-type (eql 't)))
   (declare (ignore doc-type))
-  (setf (slot-value slotd 'documentation) new-value))
+  (setf (slot-value slotd '%documentation) new-value))
 
 ;;; Now that we have created the machinery for setting documentation, we can
 ;;; set the documentation for the machinery for setting documentation.
