@@ -76,8 +76,6 @@
 
 (defgeneric class-precedence-list (pcl-class))
 
-(defgeneric class-predicate-name (class))
-
 (defgeneric class-wrapper (pcl-class))
 
 (defgeneric definition-source (definition-source-mixin))
@@ -106,7 +104,7 @@
 
 (defgeneric method-combination-options (standard-method-combination))
 
-(defgeneric method-combination-type (standard-method-combination))
+(defgeneric method-combination-type-name (standard-method-combination))
 
 (defgeneric method-fast-function (standard-method))
 
@@ -171,16 +169,16 @@
 (defgeneric (setf class-slots) (new-value slot-class))
 
 (defgeneric (setf generic-function-method-class) (new-value
-						  standard-generic-function))
+                                                  standard-generic-function))
 
 (defgeneric (setf generic-function-method-combination)
   (new-value standard-generic-function))
 
 (defgeneric (setf generic-function-declarations) (new-value
-						  standard-generic-function))
+                                                  standard-generic-function))
 
 (defgeneric (setf generic-function-methods) (new-value
-					     standard-generic-function))
+                                             standard-generic-function))
 
 (defgeneric (setf generic-function-name) (new-value standard-generic-function))
 
@@ -194,7 +192,7 @@
 (defgeneric (setf object-plist) (new-value plist-mixin))
 
 (defgeneric (setf slot-definition-allocation) (new-value
-					       standard-slot-definition))
+                                               standard-slot-definition))
 
 (defgeneric (setf slot-definition-boundp-function)
   (new-value effective-slot-definition))
@@ -222,7 +220,7 @@
 (defgeneric (setf slot-definition-name) (new-value slot-definition))
 
 (defgeneric (setf slot-definition-reader-function) (new-value
-						    effective-slot-definition))
+                                                    effective-slot-definition))
 
 (defgeneric (setf slot-definition-readers) (new-value slot-definition))
 
@@ -383,7 +381,7 @@
 
 (defgeneric update-gf-dfun (class gf))
 
-(defgeneric validate-superclass (fsc class))
+(defgeneric validate-superclass (class superclass))
 
 (defgeneric (setf documentation) (new-value slotd doc-type)
   (:argument-precedence-order doc-type slotd new-value))
@@ -410,8 +408,8 @@
 ;;; COMPUTE-EFFECTIVE-METHOD returns one value as do Allegro and
 ;;; Lispworks.
 (defgeneric compute-effective-method (generic-function
-				      combin
-				      applicable-methods))
+                                      combin
+                                      applicable-methods))
 
 (defgeneric compute-effective-slot-definition (class name dslotds))
 
@@ -438,50 +436,50 @@
 ;;;; 4 arguments
 
 (defgeneric make-method-lambda (proto-generic-function
-				proto-method
-				lambda-expression
-				environment))
+                                proto-method
+                                lambda-expression
+                                environment))
 
 (defgeneric (setf slot-value-using-class) (new-value class object slotd))
 
 ;;;; 5 arguments
 
 (defgeneric make-method-initargs-form (proto-generic-function
-				       proto-method
-				       lambda-expression
-				       lambda-list
-				       environment))
+                                       proto-method
+                                       lambda-expression
+                                       lambda-list
+                                       environment))
 
 ;;;; optional arguments
 
 (defgeneric get-method (generic-function
-			qualifiers
-			specializers
-			&optional errorp))
+                        qualifiers
+                        specializers
+                        &optional errorp))
 
 (defgeneric find-method (generic-function
-			 qualifiers
-			 specializers
-			 &optional errorp))
+                         qualifiers
+                         specializers
+                         &optional errorp))
 
 (defgeneric slot-missing (class
-			  instance
-			  slot-name
-			  operation
-			  &optional new-value))
+                          instance
+                          slot-name
+                          operation
+                          &optional new-value))
 
 ;;;; &KEY arguments
 
 (defgeneric allocate-instance (class &rest initargs))
 
 (defgeneric ensure-class-using-class (class
-				      name
-				      &rest args
-				      &key &allow-other-keys))
+                                      name
+                                      &rest args
+                                      &key &allow-other-keys))
 
 (defgeneric ensure-generic-function-using-class (generic-function
-						 fun-name
-						 &key &allow-other-keys))
+                                                 fun-name
+                                                 &key &allow-other-keys))
 
 (defgeneric initialize-instance (gf &key &allow-other-keys))
 
@@ -500,19 +498,19 @@
 (defgeneric reinitialize-instance (gf &rest args &key &allow-other-keys))
 
 (defgeneric shared-initialize (generic-function
-			       slot-names
-			       &key &allow-other-keys))
+                               slot-names
+                               &key &allow-other-keys))
 
 (defgeneric update-dependent (metaobject dependent &rest initargs))
 
 (defgeneric update-instance-for-different-class (previous
-						 current
-						 &rest initargs))
+                                                 current
+                                                 &rest initargs))
 
 (defgeneric update-instance-for-redefined-class (instance
-						 added-slots
-						 discarded-slots
-						 property-list
-						 &rest initargs))
+                                                 added-slots
+                                                 discarded-slots
+                                                 property-list
+                                                 &rest initargs))
 
 (defgeneric writer-method-class (class direct-slot &rest initargs))
