@@ -12,11 +12,13 @@
 #ifndef _ALLOC_H_
 #define _ALLOC_H_
 
+#include "sbcl.h"
 #include "runtime.h"
 
 extern lispobj alloc_cons(lispobj car, lispobj cdr);
 extern lispobj alloc_number(long n);
 extern lispobj alloc_string(char *str);
 extern lispobj alloc_sap(void *ptr);
+extern lispobj alloc_base_string(char *str);
 
-#endif _ALLOC_H_
+#endif /* _ALLOC_H_ */
