@@ -40,22 +40,22 @@ static void *compute_pc(lispobj code_obj, int pc_offset)
 
     code = (struct code *)native_pointer(code_obj);
     return (void *)((char *)code + HeaderValue(code->header)*sizeof(lispobj)
-		    + pc_offset);
+                    + pc_offset);
 }
 
-unsigned long breakpoint_install(lispobj code_obj, int pc_offset)
+unsigned int breakpoint_install(lispobj code_obj, int pc_offset)
 {
     return arch_install_breakpoint(compute_pc(code_obj, pc_offset));
 }
 
 void breakpoint_remove(lispobj code_obj, int pc_offset,
-		       unsigned long orig_inst)
+                       unsigned int orig_inst)
 {
     arch_remove_breakpoint(compute_pc(code_obj, pc_offset), orig_inst);
 }
 
 void breakpoint_do_displaced_inst(os_context_t* context,
-				  unsigned long orig_inst)
+                                  unsigned int orig_inst)
 {
     /* on platforms with sigreturn(), we go directly back from
      * arch_do_displaced_inst() to lisp code, so we need to clean up
@@ -71,7 +71,6 @@ void breakpoint_do_displaced_inst(os_context_t* context,
     arch_do_displaced_inst(context, orig_inst);
 }
 
-#if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
 static lispobj find_code(os_context_t *context)
 {
 #ifdef reg_CODE
@@ -79,133 +78,110 @@ static lispobj find_code(os_context_t *context)
     lispobj header;
 
     if (lowtag_of(code) != OTHER_POINTER_LOWTAG)
-	return NIL;
+        return NIL;
 
     header = *(lispobj *)(code-OTHER_POINTER_LOWTAG);
 
     if (widetag_of(header) == CODE_HEADER_WIDETAG)
-	return code;
+        return code;
     else
-	return code - HeaderValue(header)*sizeof(lispobj);
+        return code - HeaderValue(header)*sizeof(lispobj);
 #else
-    return NIL;
-#endif
-}
-#else
-static lispobj find_code(os_context_t *context)
-{
     lispobj codeptr =
-	(lispobj)component_ptr_from_pc((lispobj *)(*os_context_pc_addr(context)));
+        (lispobj)component_ptr_from_pc((lispobj *)(*os_context_pc_addr(context)));
 
-    if (codeptr == 0) {
-	return NIL;
-    } else {
-	return codeptr + OTHER_POINTER_LOWTAG;
-    }
-}
+    if (codeptr == 0)
+        return NIL;
+    else
+        return codeptr + OTHER_POINTER_LOWTAG;
 #endif
+}
 
 static long compute_offset(os_context_t *context, lispobj code)
 {
     if (code == NIL)
-	return 0;
+        return 0;
     else {
-	unsigned long code_start;
-	struct code *codeptr = (struct code *)native_pointer(code);
+        unsigned long code_start;
+        struct code *codeptr = (struct code *)native_pointer(code);
 #ifdef parisc
-	unsigned long pc = *os_context_pc_addr(context) & ~3;
+        unsigned long pc = *os_context_pc_addr(context) & ~3;
 #else
-	unsigned long pc = *os_context_pc_addr(context);
+        unsigned long pc = *os_context_pc_addr(context);
 #endif
 
-	code_start = (unsigned long)codeptr
-	    + HeaderValue(codeptr->header)*sizeof(lispobj);
-	if (pc < code_start)
-	    return 0;
-	else {
-	    long offset = pc - code_start;
-	    if (offset >= codeptr->code_size)
-		return 0;
-	    else
-		return make_fixnum(offset);
-	}
+        code_start = (unsigned long)codeptr
+            + HeaderValue(codeptr->header)*sizeof(lispobj);
+        if (pc < code_start)
+            return 0;
+        else {
+            long offset = pc - code_start;
+            if (offset >= codeptr->code_size)
+                return 0;
+            else
+                return make_fixnum(offset);
+        }
     }
 }
 
 void handle_breakpoint(int signal, siginfo_t* info, os_context_t *context)
 {
-    lispobj code, context_sap = alloc_sap(context);
+    lispobj code, context_sap;
 
     fake_foreign_function_call(context);
 
+    context_sap = alloc_sap(context);
     code = find_code(context);
 
+#ifndef LISP_FEATURE_WIN32
     /* Don't disallow recursive breakpoint traps. Otherwise, we can't
      * use debugger breakpoints anywhere in here. */
     thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
+#endif
 
     funcall3(SymbolFunction(HANDLE_BREAKPOINT),
-	     compute_offset(context, code),
-	     code,
-	     context_sap);
+             compute_offset(context, code),
+             code,
+             context_sap);
 
     undo_fake_foreign_function_call(context);
 }
 
-#if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
 void *handle_fun_end_breakpoint(int signal, siginfo_t *info,
-				os_context_t *context)
+                                os_context_t *context)
 {
-    lispobj code, lra;
+    lispobj code, context_sap, lra;
     struct code *codeptr;
 
     fake_foreign_function_call(context);
 
+    context_sap = alloc_sap(context);
     code = find_code(context);
     codeptr = (struct code *)native_pointer(code);
 
+#ifndef LISP_FEATURE_WIN32
     /* Don't disallow recursive breakpoint traps. Otherwise, we can't
      * use debugger breakpoints anywhere in here. */
     thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
+#endif
 
     funcall3(SymbolFunction(HANDLE_BREAKPOINT),
-	     compute_offset(context, code),
-	     code,
-	     alloc_sap(context));
+             compute_offset(context, code),
+             code,
+             context_sap);
 
     lra = codeptr->constants[REAL_LRA_SLOT];
+
 #ifdef reg_CODE
-    if (codeptr->constants[KNOWN_RETURN_P_SLOT] == NIL) {
-	*os_context_register_addr(context, reg_CODE) = lra;
-    }
+    if (codeptr->constants[KNOWN_RETURN_P_SLOT] == NIL)
+        *os_context_register_addr(context, reg_CODE) = lra;
 #endif
+
     undo_fake_foreign_function_call(context);
+
+#ifdef reg_LRA
     return (void *)(lra-OTHER_POINTER_LOWTAG+sizeof(lispobj));
-}
 #else
-void *handle_fun_end_breakpoint(int signal, siginfo_t *info,
-				os_context_t *context)
-{
-    lispobj code, context_sap = alloc_sap(context);
-    struct code *codeptr;
-
-    fake_foreign_function_call(context);
-
-    code = find_code(context);
-    codeptr = (struct code *)native_pointer(code);
-
-    /* Don't disallow recursive breakpoint traps. Otherwise, we can't
-     * use debugger breakpoints anywhere in here. */
-    thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
-
-    funcall3(SymbolFunction(HANDLE_BREAKPOINT),
-	     compute_offset(context, code),
-	     code,
-	     context_sap);
-
-    undo_fake_foreign_function_call(context);
-
-    return compute_pc(codeptr->constants[REAL_LRA_SLOT],
-		      fixnum_value(codeptr->constants[REAL_LRA_SLOT+1]));
-}
+    return compute_pc(lra, fixnum_value(codeptr->constants[REAL_LRA_SLOT+1]));
 #endif
+}
