@@ -13,6 +13,7 @@
 #define _INCLUDE_INTERRUPT_H_
 
 #include <signal.h>
+#include <string.h>
 
 /*
  * This is a workaround for some slightly silly Linux/GNU Libc
@@ -25,6 +26,8 @@
 /* FIXME: do not rely on NSIG being a multiple of 8 */
 #define REAL_SIGSET_SIZE_BYTES ((NSIG/8))
 
+extern void check_blockables_blocked_or_lose();
+
 static inline void
 sigcopyset(sigset_t *new, sigset_t *old)
 {
@@ -35,7 +38,7 @@ sigcopyset(sigset_t *new, sigset_t *old)
  *
  * Note: In CMU CL, this was 4096, but there was no explanation given,
  * and it's hard to see why we'd need that many nested interrupts, so
- * I've scaled it back (to 256) to see what happens. -- WHN 20000730 
+ * I've scaled it back (to 256) to see what happens. -- WHN 20000730
 
  * Nothing happened, so let's creep it back a bit further -- dan 20030411 */
 #define MAX_INTERRUPTS 32
@@ -45,14 +48,13 @@ union interrupt_handler {
     void (*c)(int, siginfo_t*, void*);
 };
 
-struct interrupt_data {
-    void (*interrupt_low_level_handlers[NSIG]) (int, siginfo_t*, void*) ;
-    union interrupt_handler interrupt_handlers[NSIG];
+extern union interrupt_handler interrupt_handlers[NSIG];
 
-    /* signal information for pending signal.  pending_signal=0 when there 
+struct interrupt_data {
+    /* signal information for pending signal.  pending_signal=0 when there
      * is no pending signal. */
     void (*pending_handler) (int, siginfo_t*, void*) ;
-    int pending_signal ;
+    int pending_signal;
     siginfo_t pending_info;
     sigset_t pending_mask;
 };
@@ -65,8 +67,8 @@ extern void arrange_return_to_lisp_function(os_context_t *, lispobj);
 extern void interrupt_handle_now(int, siginfo_t*, void*);
 extern void interrupt_handle_pending(os_context_t*);
 extern void interrupt_internal_error(int, siginfo_t*, os_context_t*,
-				     boolean continuable);
-extern boolean handle_guard_page_triggered(os_context_t *,void *);
+                                     boolean continuable);
+extern boolean handle_guard_page_triggered(os_context_t *,os_vm_address_t);
 extern boolean interrupt_maybe_gc(int, siginfo_t*, void*);
 extern boolean interrupt_maybe_gc_int(int, siginfo_t *, void *);
 extern boolean maybe_defer_handler(void *handler, struct interrupt_data *data,
@@ -82,17 +84,21 @@ extern void interrupt_thread_handler(int, siginfo_t*, void*);
 extern void sig_stop_for_gc_handler(int, siginfo_t*, void*);
 #endif
 extern void undoably_install_low_level_interrupt_handler (int signal,
-							  void
-							  handler(int,
-								  siginfo_t*,
-								  void*));
+                                                          void
+                                                          handler(int,
+                                                                  siginfo_t*,
+                                                                  void*));
 extern unsigned long install_handler(int signal,
-				     void handler(int, siginfo_t*, void*));
+                                     void handler(int, siginfo_t*, void*));
 
 extern union interrupt_handler interrupt_handlers[NSIG];
 
+/* Set all deferrable signals into *s. */
+extern void sigaddset_deferrable(sigset_t *s);
 /* Set all blockable signals into *s. */
-void sigaddset_blockable(sigset_t *s);
+extern void sigaddset_blockable(sigset_t *s);
+
+extern void block_blockable_signals();
 
 /* The void* casting here avoids having to mess with the various types
  * of function argument lists possible for signal handlers:
@@ -102,4 +108,3 @@ void sigaddset_blockable(sigset_t *s);
 #define ARE_SAME_HANDLER(x, y) ((void*)(x) == (void*)(y))
 
 #endif
-
