@@ -22,9 +22,9 @@
 #include <sys/syscall.h>
 #include <asm/unistd.h>
 #include <linux/version.h>
+
 #include "target-arch-os.h"
 #include "target-arch.h"
-
 #define linuxversion(a, b, c) (((a)<<16)+((b)<<8)+(c))
 
 typedef caddr_t os_vm_address_t;
@@ -40,6 +40,4 @@ typedef int os_vm_prot_t;
 
 #define SIG_INTERRUPT_THREAD (SIGRTMIN)
 #define SIG_STOP_FOR_GC (SIGRTMIN+1)
-#define SIG_DEQUEUE (SIGRTMIN+2)
-#define SIG_THREAD_EXIT (SIGRTMIN+3)
 
