@@ -15,29 +15,17 @@
  */
 
 #include <stdio.h>
-#include <sys/param.h>
-#include <sys/file.h>
-#include "sbcl.h"
-#include "./signal.h"
-#include "os.h"
-#include "arch.h"
-#include "globals.h"
-#include "interrupt.h"
-#include "interr.h"
-#include "lispregs.h"
-#include <sys/socket.h>
-#include <sys/utsname.h>
-
-#include <sys/types.h>
 #include <signal.h>
-#include <sys/time.h>
-#include <sys/stat.h>
-#include <unistd.h>
 
 /* for cacheflush() */
 #include <sys/cachectl.h>
 
-#include "validate.h"
+/* for BD_CAUSE */
+#include <asm/mipsregs.h>
+
+#include "sbcl.h"
+#include "os.h"
+#include "arch.h"
 
 size_t os_vm_page_size;
 
@@ -65,6 +53,12 @@ os_context_register_addr(os_context_t *context, int offset)
     return &(((struct sigcontext *)&(context->uc_mcontext))->sc_regs[offset]);
 }
 
+os_context_register_t *
+os_context_fpregister_addr(os_context_t *context, int offset)
+{
+    return &(((struct sigcontext *)&(context->uc_mcontext))->sc_fpregs[offset]);
+}
+
 os_context_register_t *
 os_context_pc_addr(os_context_t *context)
 {
@@ -82,9 +76,10 @@ unsigned int
 os_context_fp_control(os_context_t *context)
 {
     /* FIXME: Probably do something. */
+    return 0;
 }
 
-void 
+void
 os_restore_fp_control(os_context_t *context)
 {
     /* FIXME: Probably do something. */
@@ -101,18 +96,29 @@ os_context_bd_cause(os_context_t *context)
        loop" where a (BREAK 16) not in a branch delay slot would have
        CAUSEF_BD filled. So, we comment
 
-	#include <asm/mipsregs.h>
-
-        return (((struct sigcontext *) &(context->uc_mcontext))->sc_cause 
-	        & CAUSEF_BD);
+        return (((struct sigcontext *) &(context->uc_mcontext))->sc_cause
+                & CAUSEF_BD);
 
        out and return 0 always.  -- CSR, 2002-09-02 */
+    /* Unfortunately, returning 0 fails for taken branches because
+       os_context_bd_cause is also used to find out if a branch
+       emulation is needed.  We work around that by checking if the
+       current instruction is a jump or a branch.  */
+    extern boolean arch_insn_with_bdelay_p(unsigned int insn);
+
+    os_vm_address_t addr
+        = (os_vm_address_t)(unsigned int)*os_context_pc_addr(context);
+    unsigned int insn = *(unsigned int *)addr;
+
+    if (arch_insn_with_bdelay_p(insn))
+        return CAUSEF_BD;
+
     return 0;
 }
 
-void 
+void
 os_flush_icache(os_vm_address_t address, os_vm_size_t length)
 {
     if (cacheflush(address, length, ICACHE) == -1)
-	perror("cacheflush");
+        perror("cacheflush");
 }
