@@ -50,7 +50,7 @@ extern os_vm_size_t os_vm_page_size;
 
 /* Do anything we need to do when starting up the runtime environment
  * in this OS. */
-extern void os_init(void);
+extern void os_init(char *argv[], char *envp[]);
 
 /* Install any OS-dependent low-level signal handlers which are needed
  * by the runtime environment. E.g. the signals raised by a violation
@@ -81,9 +81,9 @@ extern void os_invalidate(os_vm_address_t addr, os_vm_size_t len);
 /* This maps a file into memory, or calls lose(..) for various
  * failures. */
 extern os_vm_address_t os_map(int fd,
-			      int offset,
-			      os_vm_address_t addr,
-			      os_vm_size_t len);
+                              int offset,
+                              os_vm_address_t addr,
+                              os_vm_size_t len);
 
 /* This presumably flushes the instruction cache, if that can be done
  * explicitly. (It doesn't seem to be an issue for the i386 port,
@@ -95,8 +95,8 @@ extern void os_flush_icache(os_vm_address_t addr, os_vm_size_t len);
  * write-protecting a page so that the garbage collector can find out
  * whether it's modified by handling the signal. */
 extern void os_protect(os_vm_address_t addr,
-		       os_vm_size_t len,
-		       os_vm_prot_t protection);
+                       os_vm_size_t len,
+                       os_vm_prot_t protection);
 
 /* This returns true for an address which makes sense at the Lisp level. */
 extern boolean is_valid_lisp_addr(os_vm_address_t test);
@@ -140,21 +140,14 @@ sigset_t *os_context_sigmask_addr(os_context_t *context);
  * depend not only on the OS, but also on the architecture, e.g.
  * getting at EFL/EFLAGS on the x86. Such things are defined in the
  * architecture-dependence files, not the OS-dependence files.) */
-   
+
 /* These are not architecture-specific functions, but are instead
  * general utilities defined in terms of the architecture-specific
  * function os_validate(..) and os_invalidate(..).
- *
- * FIXME: os_reallocate(..) is complicated and seems no longer to be
- * used for anything. Perhaps we could delete it? */
+ */
 extern os_vm_address_t os_allocate(os_vm_size_t len);
-extern os_vm_address_t os_allocate_at(os_vm_address_t addr, os_vm_size_t len);
-extern os_vm_address_t os_reallocate(os_vm_address_t addr,
-				     os_vm_size_t old_len,
-				     os_vm_size_t len);
 extern void os_deallocate(os_vm_address_t addr, os_vm_size_t len);
 
-
 /* FIXME: The os_trunc_foo(..) and os_round_foo(..) macros here could
  * be functions. */
 
@@ -179,4 +172,9 @@ extern void os_deallocate(os_vm_address_t addr, os_vm_size_t len);
  * to return the value in a way that Lisp can understand. */
 int os_get_errno(void);
 
+/* Return an absolute path to the runtime executable, or NULL if this
+ * information is unavailable.  If a non-null pathname is returned, it
+ * must be 'free'd. */
+extern char *os_get_runtime_executable_path();
+
 #endif
