@@ -5,6 +5,10 @@
 #ifndef _X86_64_ARCH_H
 #define _X86_64_ARCH_H
 
+#ifndef SBCL_GENESIS_CONFIG
+#error genesis/config.h (or sbcl.h) must be included before this file
+#endif
+
 #define ARCH_HAS_STACK_POINTER
 
 /* FIXME: Do we also want
@@ -12,17 +16,21 @@
  * here? (The answer wasn't obvious to me when merging the
  * architecture-abstracting patches for CSR's SPARC port. -- WHN 2002-02-15) */
 
-static inline void 
-get_spinlock(lispobj *word,long value)
+#include "interr.h"
+
+static inline void
+get_spinlock(volatile lispobj *word,long value)
 {
 #ifdef LISP_FEATURE_SB_THREAD
     u64 rax=0;
+    if(*word==value)
+        lose("recursive get_spinlock: 0x%x,%ld\n",word,value);
     do {
-	asm ("xor %0,%0\n\
-              lock cmpxchg %1,%2" 
-	     : "=a" (rax)
-	     : "r" (value), "m" (*word)
-	     : "memory", "cc");
+        asm ("xor %0,%0\n\
+              lock cmpxchg %1,%2"
+             : "=a" (rax)
+             : "r" (value), "m" (*word)
+             : "memory", "cc");
     } while(rax!=0);
 #else
     *word=value;
@@ -30,9 +38,20 @@ get_spinlock(lispobj *word,long value)
 }
 
 static inline void
-release_spinlock(lispobj *word)
+release_spinlock(volatile lispobj *word)
 {
     *word=0;
 }
 
+static inline lispobj
+swap_lispobjs(volatile lispobj *dest, lispobj value)
+{
+    lispobj old_value;
+    asm ("lock xchg %0,(%1)"
+         : "=r" (old_value)
+         : "r" (dest), "0" (value)
+         : "memory");
+    return old_value;
+}
+
 #endif /* _X86_64_ARCH_H */
