@@ -23,18 +23,26 @@
 #include "interrupt.h"
 #include "interr.h"
 #include "breakpoint.h"
-#include "monitor.h"
 #include "thread.h"
 
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
 
-#define BREAKPOINT_INST 0xcc	/* INT3 */
+#define BREAKPOINT_INST 0xcc    /* INT3 */
 
 unsigned long fast_random_state = 1;
 
 void arch_init(void)
 {}
+
+#ifndef LISP_FEATURE_WIN32
+os_vm_address_t
+arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
+{
+    return (os_vm_address_t)code->si_addr;
+}
+#endif
+
 
 /*
  * hacking signal contexts
@@ -46,7 +54,7 @@ void arch_init(void)
 int *
 context_eflags_addr(os_context_t *context)
 {
-#if defined __linux__
+#if defined __linux__ || defined __sun
     /* KLUDGE: As of kernel 2.2.14 on Red Hat 6.2, there's code in the
      * <sys/ucontext.h> file to define symbolic names for offsets into
      * gregs[], but it's conditional on __USE_GNU and not defined, so
@@ -57,8 +65,12 @@ context_eflags_addr(os_context_t *context)
     return &context->uc_mcontext.mc_eflags;
 #elif defined __OpenBSD__
     return &context->sc_eflags;
+#elif defined LISP_FEATURE_DARWIN
+    return &context->uc_mcontext->ss.eflags;
 #elif defined __NetBSD__
     return &(context->uc_mcontext.__gregs[_REG_EFL]);
+#elif defined LISP_FEATURE_WIN32
+    return (int *)&context->EFlags;
 #else
 #error unsupported OS
 #endif
@@ -74,38 +86,38 @@ void arch_skip_instruction(os_context_t *context)
     int vlen;
     int code;
 
-    
+
     /* Get and skip the Lisp interrupt code. */
     code = *(char*)(*os_context_pc_addr(context))++;
     switch (code)
-	{
-	case trap_Error:
-	case trap_Cerror:
-	    /* Lisp error arg vector length */
-	    vlen = *(char*)(*os_context_pc_addr(context))++;
-	    /* Skip Lisp error arg data bytes. */
-	    while (vlen-- > 0) {
-		++*os_context_pc_addr(context);
-	    }
-	    break;
+        {
+        case trap_Error:
+        case trap_Cerror:
+            /* Lisp error arg vector length */
+            vlen = *(char*)(*os_context_pc_addr(context))++;
+            /* Skip Lisp error arg data bytes. */
+            while (vlen-- > 0) {
+                ++*os_context_pc_addr(context);
+            }
+            break;
 
-	case trap_Breakpoint:		/* not tested */
-	case trap_FunEndBreakpoint: /* not tested */
-	    break;
+        case trap_Breakpoint:           /* not tested */
+        case trap_FunEndBreakpoint: /* not tested */
+            break;
 
-	case trap_PendingInterrupt:
-	case trap_Halt:
-	    /* only needed to skip the Code */
-	    break;
+        case trap_PendingInterrupt:
+        case trap_Halt:
+            /* only needed to skip the Code */
+            break;
 
-	default:
-	    fprintf(stderr,"[arch_skip_inst invalid code %d\n]\n",code);
-	    break;
-	}
+        default:
+            fprintf(stderr,"[arch_skip_inst invalid code %d\n]\n",code);
+            break;
+        }
 
     FSHOW((stderr,
-	   "/[arch_skip_inst resuming at %x]\n",
-	   *os_context_pc_addr(context)));
+           "/[arch_skip_inst resuming at %x]\n",
+           *os_context_pc_addr(context)));
 }
 
 unsigned char *
@@ -124,26 +136,33 @@ void
 arch_set_pseudo_atomic_interrupted(os_context_t *context)
 {
     SetSymbolValue(PSEUDO_ATOMIC_INTERRUPTED, make_fixnum(1),
-		   arch_os_get_current_thread());
+                   arch_os_get_current_thread());
+}
+
+void
+arch_clear_pseudo_atomic_interrupted(os_context_t *context)
+{
+    SetSymbolValue(PSEUDO_ATOMIC_INTERRUPTED, make_fixnum(0),
+                   arch_os_get_current_thread());
 }
 
 /*
  * This stuff seems to get called for TRACE and debug activity.
  */
 
-unsigned long
+unsigned int
 arch_install_breakpoint(void *pc)
 {
-    unsigned long result = *(unsigned long*)pc;
+    unsigned int result = *(unsigned int*)pc;
 
-    *(char*)pc = BREAKPOINT_INST;		/* x86 INT3       */
-    *((char*)pc+1) = trap_Breakpoint;		/* Lisp trap code */
+    *(char*)pc = BREAKPOINT_INST;               /* x86 INT3       */
+    *((char*)pc+1) = trap_Breakpoint;           /* Lisp trap code */
 
     return result;
 }
 
 void
-arch_remove_breakpoint(void *pc, unsigned long orig_inst)
+arch_remove_breakpoint(void *pc, unsigned int orig_inst)
 {
     *((char *)pc) = orig_inst & 0xff;
     *((char *)pc + 1) = (orig_inst & 0xff00) >> 8;
@@ -180,7 +199,7 @@ arch_do_displaced_inst(os_context_t *context, unsigned int orig_inst)
     *context_eflags_addr(context) |= 0x100;
 #endif
 
-    single_stepping = (unsigned int*)pc;
+    single_stepping = pc;
 
 #ifdef CANNOT_GET_TO_SINGLE_STEP_FLAG
     *os_context_pc_addr(context) = (char *)pc - 9;
@@ -190,38 +209,39 @@ arch_do_displaced_inst(os_context_t *context, unsigned int orig_inst)
 void
 sigtrap_handler(int signal, siginfo_t *info, void *void_context)
 {
-    int code = info->si_code;
     os_context_t *context = (os_context_t*)void_context;
     unsigned int trap;
 
+#ifndef LISP_FEATURE_WIN32
     if (single_stepping && (signal==SIGTRAP))
     {
-	/* fprintf(stderr,"* single step trap %x\n", single_stepping); */
+        /* fprintf(stderr,"* single step trap %x\n", single_stepping); */
 
 #ifdef CANNOT_GET_TO_SINGLE_STEP_FLAG
-	/* Un-install single step helper instructions. */
-	*(single_stepping-3) = single_step_save1;
-	*(single_stepping-2) = single_step_save2;
-	*(single_stepping-1) = single_step_save3;
+        /* Un-install single step helper instructions. */
+        *(single_stepping-3) = single_step_save1;
+        *(single_stepping-2) = single_step_save2;
+        *(single_stepping-1) = single_step_save3;
 #else
-	*context_eflags_addr(context) ^= 0x100;
+        *context_eflags_addr(context) &= ~0x100;
 #endif
-	/* Re-install the breakpoint if possible. */
-	if (*os_context_pc_addr(context) == (int)single_stepping + 1) {
-	    fprintf(stderr, "warning: couldn't reinstall breakpoint\n");
-	} else {
-	    *((char *)single_stepping) = BREAKPOINT_INST;	/* x86 INT3 */
-	    *((char *)single_stepping+1) = trap_Breakpoint;
-	}
+        /* Re-install the breakpoint if possible. */
+        if (*os_context_pc_addr(context) == (int)single_stepping + 1) {
+            fprintf(stderr, "warning: couldn't reinstall breakpoint\n");
+        } else {
+            *((char *)single_stepping) = BREAKPOINT_INST;       /* x86 INT3 */
+            *((char *)single_stepping+1) = trap_Breakpoint;
+        }
 
-	single_stepping = NULL;
-	return;
+        single_stepping = NULL;
+        return;
     }
+#endif
 
     /* This is just for info in case the monitor wants to print an
      * approximation. */
     current_control_stack_pointer =
-	(lispobj *)*os_context_sp_addr(context);
+        (lispobj *)*os_context_sp_addr(context);
 
     /* FIXME: CMUCL puts the float control restoration code here.
        Thus, it seems to me that single-stepping won't restore the
@@ -229,61 +249,85 @@ sigtrap_handler(int signal, siginfo_t *info, void *void_context)
        single-stepping (as far as I can tell) this is somewhat moot,
        but it might be worth either moving this code up or deleting
        the single-stepping code entirely.  -- CSR, 2002-07-15 */
-#ifdef LISP_FEATURE_LINUX
+#if defined(LISP_FEATURE_LINUX) || defined(RESTORE_FP_CONTROL_FROM_CONTEXT)
     os_restore_fp_control(context);
 #endif
 
+
+#ifdef LISP_FEATURE_SUNOS
+    /* For some reason the breakpoints that :ENCAPSULATE NIL tracing sets up
+     * cause a trace trap (i.e. processor single-stepping trap) on the following
+     * instruction on Solaris 10/x86. -- JES, 2006-04-07
+     */
+    if (info->si_code == TRAP_TRACE) {
+        lose("foo");
+        return;
+    }
+#endif
+
     /* On entry %eip points just after the INT3 byte and aims at the
      * 'kind' value (eg trap_Cerror). For error-trap and Cerror-trap a
      * number of bytes will follow, the first is the length of the byte
      * arguments to follow. */
     trap = *(unsigned char *)(*os_context_pc_addr(context));
+    /* FSHOW((stderr, "/<sigtrap trap %d at pc_addr: %p>\n", trap, *os_context_pc_addr(context))); */
     switch (trap) {
 
     case trap_PendingInterrupt:
-	FSHOW((stderr, "/<trap pending interrupt>\n"));
-	arch_skip_instruction(context);
+        FSHOW((stderr, "/<trap pending interrupt>\n"));
+        arch_skip_instruction(context);
         interrupt_handle_pending(context);
-	break;
+        break;
 
     case trap_Halt:
-	/* Note: the old CMU CL code tried to save FPU state
-	 * here, and restore it after we do our thing, but there
-	 * seems to be no point in doing that, since we're just
-	 * going to lose(..) anyway. */
-	fake_foreign_function_call(context);
-	lose("%%PRIMITIVE HALT called; the party is over.");
+        /* Note: the old CMU CL code tried to save FPU state
+         * here, and restore it after we do our thing, but there
+         * seems to be no point in doing that, since we're just
+         * going to lose(..) anyway. */
+        fake_foreign_function_call(context);
+        lose("%%PRIMITIVE HALT called; the party is over.\n");
 
     case trap_Error:
     case trap_Cerror:
-	FSHOW((stderr, "<trap error/cerror %d>\n", code));
-	interrupt_internal_error(signal, info, context, code==trap_Cerror);
-	break;
+        FSHOW((stderr, "<trap error/cerror %d>\n", trap));
+        interrupt_internal_error(signal, info, context, trap==trap_Cerror);
+        break;
 
     case trap_Breakpoint:
-	--*os_context_pc_addr(context);
-	handle_breakpoint(signal, info, context);
-	break;
+        --*os_context_pc_addr(context);
+        handle_breakpoint(signal, info, context);
+        break;
 
     case trap_FunEndBreakpoint:
-	--*os_context_pc_addr(context);
-	*os_context_pc_addr(context) =
-	    (int)handle_fun_end_breakpoint(signal, info, context);
-	break;
+        --*os_context_pc_addr(context);
+        *os_context_pc_addr(context) =
+            (int)handle_fun_end_breakpoint(signal, info, context);
+        break;
 
     default:
-	FSHOW((stderr,"/[C--trap default %d %d %x]\n",
-	       signal, code, context));
-	interrupt_handle_now(signal, info, context);
-	break;
+        FSHOW((stderr,"/[C--trap default %d %d %x]\n",
+               signal, trap, context));
+        interrupt_handle_now(signal, info, context);
+        break;
     }
 }
 
 static void
 sigill_handler(int signal, siginfo_t *siginfo, void *void_context) {
     os_context_t *context = (os_context_t*)void_context;
+
+    /* Triggering SIGTRAP using int3 is unreliable on OS X/x86, so
+     * we need to use illegal instructions for traps.
+     */
+#if defined(LISP_FEATURE_DARWIN)
+    if (*((unsigned short *)*os_context_pc_addr(context)) == 0x0b0f) {
+        *os_context_pc_addr(context) += 2;
+        return sigtrap_handler(signal, siginfo, void_context);
+    }
+#endif
+
     fake_foreign_function_call(context);
-    monitor_or_something();
+    lose("fake_foreign_call fell through");
 }
 
 void
@@ -301,8 +345,10 @@ arch_install_interrupt_handlers()
      * OS I haven't tested on?) and we have to go back to the old CMU
      * CL way, I hope there will at least be a comment to explain
      * why.. -- WHN 2001-06-07 */
+#ifndef LISP_FEATURE_WIN32
     undoably_install_low_level_interrupt_handler(SIGILL , sigill_handler);
     undoably_install_low_level_interrupt_handler(SIGTRAP, sigtrap_handler);
+#endif
 
     SHOW("returning from arch_install_interrupt_handlers()");
 }
@@ -358,7 +404,7 @@ funcall3(lispobj function, lispobj arg0, lispobj arg1, lispobj arg2)
  * things.
  */
 
-void 
+void
 arch_write_linkage_table_jmp(char * reloc, void * fun)
 {
     /* Make JMP to function entry. JMP offset is calculated from next
@@ -367,10 +413,10 @@ arch_write_linkage_table_jmp(char * reloc, void * fun)
     long offset = (char *)fun - (reloc + 5);
     int i;
 
-    *reloc++ = 0xe9;		/* opcode for JMP rel32 */
+    *reloc++ = 0xe9;            /* opcode for JMP rel32 */
     for (i = 0; i < 4; i++) {
-	*reloc++ = offset & 0xff;
-	offset >>= 8;
+        *reloc++ = offset & 0xff;
+        offset >>= 8;
     }
 
     /* write a nop for good measure. */
