@@ -6,7 +6,7 @@
 ;;;; While most of SBCL is derived from the CMU CL system, the test
 ;;;; files (like this one) were written from scratch after the fork
 ;;;; from CMU CL.
-;;;; 
+;;;;
 ;;;; This software is in the public domain and is provided with
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
@@ -116,8 +116,8 @@
   t)
 
 (defun-with-dx dxclosure (x)
-  (flet ((f (y) 
-	   (+ y x)))
+  (flet ((f (y)
+           (+ y x)))
     (declare (dynamic-extent #'f))
     (true #'f)))
 
@@ -134,7 +134,7 @@
       (funcall thunk))
     (assert (< (- (get-bytes-consed) before) times))))
 
-#+(or x86 x86-64 alpha ppc sparc)
+#+(or x86 x86-64 alpha ppc sparc mips)
 (progn
   (assert-no-consing (dxclosure 42))
   (assert-no-consing (dxlength 1 2 3))
@@ -188,4 +188,3 @@
 (bdowning-2005-iv-16)
 
 
-(sb-ext:quit :unix-status 104)
