@@ -6,7 +6,7 @@
 ;;;; While most of SBCL is derived from the CMU CL system, the test
 ;;;; files (like this one) were written from scratch after the fork
 ;;;; from CMU CL.
-;;;; 
+;;;;
 ;;;; This software is in the public domain and is provided with
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
@@ -22,4 +22,3 @@
   (package-error (c) (princ c))
   (:no-error (&rest args) (error "(EXPORT :FOO) returned ~S" args)))
 
-(sb-ext:quit :unix-status 104)
